C
C  Main Program for chi_auto - contains all subroutines for simple
C  debugging
C
C  RWP - 1992 May 19
C
C---------------------------------------------------------------------------

      program chi_auto
 
C  Automatic scaling of spectra assuming constant flux in certain lines.
C
C  Version 1.0, August 1991
C
C  Ernst van Groningen, Ignaz Wanders
C  Uppsala Astronomical Observatory
C  Box 515
C  S-75120 Uppsala
C  Sweden
C     e-mail:
C       21619::LABAN::ERNST (SPAN)  Ernst.van_Groningen@astro.uu.se  (INTERNET)
C       21619::LABAN::IGNAZ (SPAN)  Ignaz.Wanders@astro.uu.se        (INTERNET)
C
C  The programme fits a simple function to the difference between the
C  2 spectra in the specified wavelength range, and calculates the
C  chisq of this fit. By varying the flux factor, wavelength shift and
C  FWHM of the resolution of the second spectrum the minimum chisq is
C  searched for, according to a grid search method (see Bevington, Data
C  reduction and error analysis for the physical sciences, McGraw-Hill,
C  1969, page 208ff). The search is performed twice, either convolving
C  the first or the second spectrum. The lowest value found is used to
C  calculate the output file. The programme stops when the relative change
C  in chisq from one step to the next is less than the "stopping criterium"
C
C  The programme prompts for the following input parameters
C        > reference spectrum with 3 columns, lambda,flux,rms
C        > spectrum to be scaled, same format
C        > start wavelength of fitting region
C        > end wavelength of fitting region
C        > initial estimate of flux scale factor
C        > initial estimate of FWHM of spectral convolution (pixels)
C        > initial estimate of wavelength shift (pixels)
C        > step size in flux factor
C        > step size in FWHM
C        > step size in shift
C        > stopping criterium
C        > name of output file name
C
C  A summary of the results is written to the file 'results' (in
C  routine WRITE_RES). The 4 columns contain:
C        > the flux ratio (i.e. for scaling the second input file)
C        > the FWHM ( if FWHM>0 the second file is convolved, if
C            FWHM<0 the first file)
C        > the wavelength shift (applied on the second file)
C        > the value of chisq in the minimum
C
C  The modified spectra are written to the specified output file which
C  contains seven columns, whose contents depend on the sign of the
C  FWHM (see above)
C
C  For FWHM > 0
C        1. wavelength
C        2. second spectrum: scaled, shifted and convolved
C        3. one sigma rms of column 2
C        4. identical to column 2
C        5. second spectrum: scaled, shifted (not convolved)
C        6. difference between reference spectrum and column 2
C        7. difference between reference spectrum and column 5
C
C  For FWHM < 0
C        1. wavelength
C        2. second spectrum: scaled, shifted (not convolved)
C        3. one sigma rms of column 2
C        4. convolved reference spectrum
C        5. non modified reference spectrum
C        6. difference between column 4 and column 2
C        7. difference between column 5 and column 2
C
C  CPU time usage depends very much on how good the initial guesses are and
C  on the step sizes. Some tests with 'typical' input parameters yielded:
C
C       Apollo workstation  -	DN3550    140 secs
C				DN10000     4 secs
C	VAX 4300		           16 secs
C
C  Modified to be explicitly ANSI-standard Fortran 77  [RWP/OSU 1992 Apr 29]
C
 
c     implicit none
 
      integer nmax, ntot, status

C  pixel numbers of starting and end of fitting region

      integer n1,n2    

C  maximum spectral length 

      parameter        (NMAX=1024)

C  Array of wavelengths
 
      real wave(NMAX)

C  original flux and sigmas of first file

      real flux_1(NMAX), sig_1(NMAX)

C  original flux and sigmas of second file

      real flux_2(NMAX), sig_2(NMAX)

C  starting values for flux ratio, FWHM and lambda shift

      real r0, f0, d0

C  initial step size for these three parameters

      real dr, df, dd

C  results for these three parameters

      real r, f, d

C  stop if delta(chisq) < halt * ntot

      real halt

C  Chi-squared

      real chisq
 
C-----------------------------------------------------------------------
C    Read spectra from disk
 
      call read_data(wave,flux_1,sig_1,flux_2,sig_2,ntot)
 
C-----------------------------------------------------------------------
C    Inquire for lambda region and find start and end pixel for extraction
 
      call read_lamb(wave,ntot,n1,n2)
 
C-----------------------------------------------------------------------
C    Inquire for starting values, initial stepsizes and stopping criterium
 
      call read_init(r0,f0,d0,dr,df,dd,halt)
 
C-----------------------------------------------------------------------
C    Locate the position of the minimum chisq
 
      call find_min(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &              r0,f0,d0,dr,df,dd,halt,r,f,d,chisq,status)
 
 
C-----------------------------------------------------------------------
C    Output the position of the minimum and the value of chisq
 
      call write_res(r,f,d,chisq)
 
C-----------------------------------------------------------------------
C    Output the resulting spectra to disk
 
      call wrout(wave,flux_1,sig_1,flux_2,sig_2,ntot,r,f,d)

      print *,'*** CHI_AUTO Done ***'
      end

C---------------------------------------------------------------------------

      subroutine conv_gauss (in,out,sigma,n,flag)
 
      real    in(n), out(n), gauss(1000), res(1000)
      real    sum, sigma
      integer n, nelm2, i, j, nc, nout
      logical flag

C 
C    Subroutine to convolve array "in" with a gaussian with width "sigma"
C    In case the flag is .true. this means we are convolving an error array
C    and we need to square everything before convolving and to take the
C    square root after convolution.
C
 
C ------------------------------------------------------------------------

      if (sigma .eq. 0.0) then
         do 100 i = 1,n
            out(i) = in(i)
 100     continue
         return
      end if
 
      nc = max( int(0.5*sigma), 5)
      nelm2 = 2*nc-1
 
      sum = 0.0
      do 200 i = 1,nelm2
         gauss(i) = exp(-0.5*((i-nc)/sigma)**2)
         sum = sum + gauss(i)
 200  continue
      do 300 i = 1, nelm2
         gauss(i) = gauss(i)/sum
 300  continue
 
      nout = n - nelm2 + 1
 
C ------------------------------------------------------------------------
C     Do the convolution

C   Flux Convolution
 
      if (.not. flag) then
         do 400 i = 1,nout
            sum = 0.0
            do 500 j = 1,nelm2
               sum = sum + in(i+j-1) * gauss(nelm2-j+1)
 500        continue
            res(i) = sum
 400     continue

C   Sigma Convolution

      else
         do 600 i = 1,nout
            sum = 0.0
            do 700 j = 1,nelm2
               sum = sum + in(i+j-1)**2 * gauss(nelm2-j+1)**2
 700        continue
            res(i) = sqrt(sum)
 600     continue
      end if
 
      do 800 i = 1,(nelm2-1)/2
         out(i) = in(i)
 800  continue
 
      do 900 i = 1,nout
         out((nelm2-1)/2+i) = res(i)
 900  continue
 
      do 1000 i = nout+1,n
         out(i) = in(i)
 1000 continue

      return
      end

C---------------------------------------------------------------------------

      subroutine covsrt(covar,ncvm,ma,lista,mfit)

C
C   Given a covariance matrix COVAR of a fit for MFIT of MA total parameters,
C   and their ordering LISTA(I), repack the covariance matrix to the true
C   order of the parameters.  Elements associated with fixed parameters will
C   be zero.  NCVM is the physical dimension of COVAR
C
C   Taken from Numerical Recipes (Press, Flannery, Teukolsky, & Vetterling), 
C   Chapter 14.3
C

      real    covar(ncvm,ncvm)
      integer lista(mfit)

      do 12 j = 1, ma-1
         do 11 i = j+1, ma
            covar(i,j) = 0.
 11      continue
 12   continue
      do 14 i = 1, mfit-1
         do 13 j = i+1, mfit
            if (lista(j) .gt. lista(i)) then
               covar(lista(j),lista(i)) = covar(i,j)
            else
               covar(lista(i),lista(j)) = covar(i,j)
            end if
 13      continue
 14   continue
      swap = covar(1,1)
      do 15 j = 1, ma
         covar(1,j) = covar(j,j)
         covar(j,j) = 0.
 15   continue
      covar(lista(1),lista(1)) = swap
      do 16 j = 2, mfit
         covar(lista(j),lista(j)) = covar(1,j)
 16   continue
      do 18 j = 2, ma
         do 17 i = 1,j-1
            covar(i,j) = covar(j,i)
 17      continue
 18   continue
      return
      end
      
C---------------------------------------------------------------------------

      subroutine find_min(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &                    r0,f0,d0,dr,df,dd,halt,r,f,d,chisq0,stat)
 
C  Declarations
 
      integer n1,n2,ntot,max_iter,count,stat
      real wave(ntot)
      real flux_1(ntot),sig_1(ntot)
      real flux_2(ntot),sig_2(ntot)
      real r0,f0,d0,dr,df,dd,r,f,d,halt,chisq
      real rnew,fnew,dnew,chisq0
      real rtemp,ftemp,dtemp,chitemp
      logical proceed
 
      parameter (MAX_ITER=30)
 
C  Minimum found if stat not changed

      stat = 0
      count = 0
 
      proceed = .true.
      r = r0
      f = f0
      d = d0
 
      chisq0 = 1e6
 
C-----------------------------------------------------------------------
C   Start the grid search
 
      do while (proceed)
         count = count + 1
 
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying r and constant f and d.
 
         call find_min_r(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &                   r,f,d,dr,rnew,chisq)
         r = rnew
 
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying d and constant r and f.
 
         call find_min_d(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &        r,f,d,dd,dnew,chisq)
         d = dnew
 
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying f and constant r and d.
 
         call find_min_f(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &        r,f,d,df,fnew,chisq)
         f = fnew
 
C-----------------------------------------------------------------------
 
         if (abs(chisq-chisq0).lt.halt*ntot) proceed = .false.
         if (count.ge.MAX_ITER) then
            proceed = .false.
            stat=1
         end if
         chisq0 = chisq
      end do
 
C-----------------------------------------------------------------------
C   Store the values of r, f, d and chisq and interchange the files
C   to check if a lower minimum is found when the other file is
C   convolved
 
      rtemp = r
      ftemp = f
      dtemp = d
      chitemp = chisq0
 
C-----------------------------------------------------------------------
C   Repeat procedure with the files interchanged
 
      count = 0
 
      proceed = .true.
      r = r0
      f = f0
      d = d0
 
      chisq0 = 1.0e6
 
C-----------------------------------------------------------------------
C   Start the grid search
 
      do while (proceed)
         count = count + 1
 
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying r and constant f and d.
 
         call find_min_r(wave,flux_2,sig_2,flux_1,sig_1,ntot,n1,n2,
     &        r,f,d,dr,rnew,chisq)
         r = rnew
            
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying d and constant r and f.
 
         call find_min_d(wave,flux_2,sig_2,flux_1,sig_1,ntot,n1,n2,
     &        r,f,d,dd,dnew,chisq)
         d = dnew
 
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying f and constant r and d.
 
         call find_min_f(wave,flux_2,sig_2,flux_1,sig_1,ntot,n1,n2,
     &        r,f,d,df,fnew,chisq)
         f = fnew
 
C-----------------------------------------------------------------------
 
         if (abs(chisq-chisq0).lt.halt*ntot) then
            proceed = .false.
            stat = 0
         end if
         if (count .ge. MAX_ITER) then
            proceed = .false.
         end if
         chisq0 = chisq
      end do
 
      if (chitemp.lt.chisq) then
         r = rtemp
         f = ftemp
         d = dtemp
         chisq0 = chitemp
      else
         f = -f
         d = -d
         r = 1/r
      end if
 
      return
      end

C---------------------------------------------------------------------------

      subroutine find_min_D(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &                      r,f,d,dd,dnew,chisq)
 
C     implicit none
 
      integer NMAX,ntot,i,j,mpoly,n1,n2,count,max_iter
      
C  # of coefficients in fitting polynomial
      parameter (MPOLY=3)
      parameter (NMAX=1024)
C  maximum iterations
      parameter (MAX_ITER=50)
 
 
      real wave(ntot),wave1(NMAX)
      real flux_1(ntot),sig_1(ntot)
      real flux_2(ntot),sig_2(ntot)
      real dd,r,f,d,chisq,dnew,shift
      real work1(NMAX),work2(NMAX),work3(NMAX),work4(NMAX)
      real diff(NMAX),sigma(NMAX)
      real coeff(MPOLY), chi(3)
 
      count = 0
 
C-----------------------------------------------------------------------
C    Put the scaled and convolved spectra into work1 and work2
      
C-----------------------------------------------------------------------
C   Apply convolution with FWHM = "f"
 
      call conv_gauss(flux_2,work1,f/2.35482,ntot,.false.)
      call conv_gauss(sig_2,work2,f/2.35482,ntot,.true.)
 
C-----------------------------------------------------------------------
C   Scale spectra by present flux factor
 
      do 100 i = 1,ntot
         work1(i) = work1(i)*r
         work2(i) = work2(i)*r
 100  continue
 
C-----------------------------------------------------------------------
C   Fill the wavelength work array
 
      do 110 i = n1,n2
         wave1(i-n1+1) = wave(i)
 110  continue
 
C-----------------------------------------------------------------------
C   Find chisq at three positions d-dd,d and d+dd
 
 1    do 200 j = 1,3
         shift = d+(j-2)*dd
C-----------------------------------------------------------------------
C   Fill the arrays wave1,diff and sigma with wavelengths, difference
C   and sigma's resp. for the required wavelength range
 
         call shift_spec(work1,work3,ntot,shift,.false.)
         call shift_spec(work2,work4,ntot,shift,.true.)
 
         do 210 i = n1,n2
            diff(i-n1+1) = flux_1(i) - work3(i)
            sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 210     continue
 
C-----------------------------------------------------------------------
C   Do the polynomial fitting and calculate chisq
 
         call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
         chi(j) = chisq
 200  continue
 
C-----------------------------------------------------------------------
C   Find minimum in chisq
 
 2    count = count + 1

C*      write(6,98) shift,chi(1),chi(2),chi(3)
C*98   format(' Scanning D: Shift = ',f10.5,' Chisq = ',3f12.2)

      if (count .ge. MAX_ITER) then
         dnew = shift
         goto 4
      end if
      if (count .ge. MAX_ITER) goto 3
      if (chi(2) .le. chi(1)) then
         if (chi(3) .lt. chi(2)) then
 
C-----------------------------------------------------------------------
C   Chisq decreases with increasing ratio
 
            chi(1) = chi(2)
            chi(2) = chi(3)
            shift = shift+dd
            call shift_spec(work1,work3,ntot,shift,.false.)
            call shift_spec(work2,work4,ntot,shift,.true.)
            do 300 i = n1,n2
               diff(i-n1+1) = flux_1(i) - work3(i)
               sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 300        continue
            call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
            chi(3) = chisq
            goto 2
         else
 
C-----------------------------------------------------------------------
C   Minimum found !!!

            goto 3
         end if
      else
         if (chi(3) .ge. chi(2)) then
 
C-----------------------------------------------------------------------
C   Chisq increases with increasing shift
 
            chi(3) = chi(2)
            chi(2) = chi(1)
            shift = shift - 3*dd
            call shift_spec(work1,work3,ntot,shift,.false.)
            call shift_spec(work2,work4,ntot,shift,.true.)
            do 400 i = n1,n2
               diff(i-n1+1) = flux_1(i) - work3(i)
               sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 400        continue
            call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
            chi(1) = chisq
            shift = shift + 2*dd
            goto 2
         else
 
C-----------------------------------------------------------------------
C   Oops, local maximum in chisq. Choose the direction of the lowest and
C      start from the beginning.
 
            if (chi(3) .lt. chi(1)) d = d + dd
            if (chi(3) .gt. chi(1)) d = d - dd
            goto 1
         end if
      end if
 
C-----------------------------------------------------------------------
C   Calculate the actual position of the minimum in chisq by a parabolic
C   fit to the last three values in chi (Bevington, p210).
C   (Where a(3) = shift)
 
 3    dnew = shift - dd * (0.5 + (chi(3)-chi(2))/
     &       (chi(3)-2*chi(2)+chi(1)))
 
C-----------------------------------------------------------------------
C   Calculate the actual minimum chisq
 
 4    call shift_spec(work1,work3,ntot,dnew,.false.)
      call shift_spec(work2,work4,ntot,dnew,.true.)
      do 500 i = n1,n2
         diff(i-n1+1) = flux_1(i) - work3(i)
         sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 500  continue
      call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)

C*       write(6,99) dnew, chisq
C* 099   format(' Found minimum chisq in D: Shift = ',f10.5,' Chisq = ',
C*    >              f12.2)
 
      return
      end

C---------------------------------------------------------------------------

      subroutine find_min_f(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &                      r,f,d,df,fnew,chisq)
 
c     implicit none
 
      integer NMAX,ntot,i,j,mpoly,n1,n2,max_iter,count
 
C  # of coefficients in fitting polynomial
      parameter (MPOLY=3)
      parameter (NMAX=1024)
      parameter (MAX_ITER=50)
 
      real wave(ntot),wave1(NMAX)
      real flux_1(ntot),sig_1(ntot)
      real flux_2(ntot),sig_2(ntot)
      real df,r,f,d,chisq,fnew,FWHM
      real work1(NMAX),work2(NMAX),work3(NMAX),work4(NMAX)
      real diff(NMAX),sigma(NMAX)
      real coeff(MPOLY), chi(3)
 
      count = 0
 
C-----------------------------------------------------------------------
C    Put the scaled and shifted spectra into work1 and work2
 
C-----------------------------------------------------------------------
C   Apply shift by "d" pixels
 
      call shift_spec(flux_2,work1,ntot,d,.false.)
      call shift_spec(sig_2,work2,ntot,d,.true.)
 
C-----------------------------------------------------------------------
C   Scale spectra by present flux factor
 
      do 100 i = 1,ntot
         work1(i) = work1(i)*r
         work2(i) = work2(i)*r
 100  continue
 
C-----------------------------------------------------------------------
C   Fill the wavelength work array
 
      do 110 i = n1,n2
         wave1(i-n1+1) = wave(i)
 110  continue
 
C-----------------------------------------------------------------------
C   Find chisq at three positions f,f+df,f+2df
 
 1    do 200 j = 1,3
         FWHM = f+(j-1)*df

C-----------------------------------------------------------------------
C   Fill the arrays wave1,diff and sigma with wavelengths, difference
C   and sigma's resp. for the required wavelength range
 
         call conv_gauss(work1,work3,FWHM/2.35482,ntot,.false.)
         call conv_gauss(work2,work4,FWHM/2.35482,ntot,.true.)
 
         do 210 i = n1,n2
            diff(i-n1+1) = flux_1(i) - work3(i)
            sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 210     continue
 
C-----------------------------------------------------------------------
C   Do the polynomial fitting and calculate chisq
 
         call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
         chi(j) = chisq
 200  continue
 
C-----------------------------------------------------------------------
C   Find minimum in chisq
 
 2    count = count + 1

C*       write(6,98) FWHM,chi(1),chi(2),chi(3)
C* 98   format(' Scanning F: FWHM  = ',f10.5,' Chisq = ',3f12.2)

      if (count .ge. MAX_ITER) then
         fnew = FWHM
         goto 4
      end if
      if (chi(2) .le. chi(1)) then
         if (chi(3) .lt. chi(2)) then
 
C-----------------------------------------------------------------------
C   Chisq decreases with increasing FWHM
 
            chi(1) = chi(2)
            chi(2) = chi(3)
            FWHM = FWHM+df
            call conv_gauss(work1,work3,FWHM/2.35482,ntot,.false.)
            call conv_gauss(work2,work4,FWHM/2.35482,ntot,.true.)
            do 300 i = n1,n2
               diff(i-n1+1) = flux_1(i) - work3(i)
               sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 300        continue
            call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
            chi(3) = chisq
            goto 2
         else
 
C-----------------------------------------------------------------------
C   Minimum found !!!

            goto 3
         end if
      else
         if (chi(3) .ge. chi(2)) then
 
C-----------------------------------------------------------------------
C   Chisq increases with decreasing FWHM
 
            chi(3) = chi(2)
            chi(2) = chi(1)
            FWHM = FWHM - 3*df
            call conv_gauss(work1,work3,FWHM/2.35482,ntot,.false.)
            call conv_gauss(work2,work4,FWHM/2.35482,ntot,.true.)
            do 400 i = n1,n2
               diff(i-n1+1) = flux_1(i) - work3(i)
               sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 400        continue
            call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
            chi(1) = chisq
            FWHM = FWHM + 2*df
            goto 2
         else
 
C-----------------------------------------------------------------------
C   Oops, local maximum in chisq. Choose the direction of the lowest and
C      start from the beginning.
 
            if (chi(3) .lt. chi(1)) f = f + df
            if (chi(3) .gt. chi(1)) f = f - df
            goto 1
         end if
      end if
 
C-----------------------------------------------------------------------
C   Calculate the actual position of the minimum in chisq by a parabolic
C   fit to the last three values in chi (Bevington, p210).
C   (Where a(3) = FWHM)
 
 3    fnew = FWHM - df * (0.5 + (chi(3)-chi(2))/
     &                   (chi(3)-2*chi(2)+chi(1)))
 
C-----------------------------------------------------------------------
C   Calculate the actual minimum chisq
 
 4    call conv_gauss(work1,work3,fnew/2.35482,ntot,.false.)
      call conv_gauss(work2,work4,fnew/2.35482,ntot,.true.)
 
      do 500 i = n1,n2
         diff(i-n1+1) = flux_1(i) - work3(i)
         sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 500  continue
 
      call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)

C*       write(6,99) fnew, chisq
C* 099   format(' Found minimum chisq in F: FWHM  = ',f10.5,' Chisq = ',
C*    >              f12.2)
 
      return
      end

C---------------------------------------------------------------------------

      subroutine find_min_r(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &                      r,f,d,dr,rnew,chisq)
 
c     implicit none
 
      integer NMAX,ntot,i,j,MPOLY,n1,n2,MAX_ITER,count

C  # of coefficients in fitting polynomial
      parameter (MPOLY=3)
      parameter (NMAX=1024)
C  maximum iterations
      parameter (MAX_ITER=50)
 
      real wave(ntot),wave1(NMAX)
      real flux_1(ntot),sig_1(ntot)
      real flux_2(ntot),sig_2(ntot)
      real dr,r,f,d,chisq,rnew,ratio
      real work1(NMAX),work2(NMAX),work3(NMAX),work4(NMAX)
      real diff(NMAX),sigma(NMAX)
      real coeff(MPOLY), chi(3)
 
      count = 0
 
C-----------------------------------------------------------------------
C    Put the convolved and shifted spectra into work3 and work4
 
C-----------------------------------------------------------------------
C   Apply convolution with FWHM = "f"
 
      call conv_gauss(flux_2,work1,f/2.35482,ntot,.false.)
      call conv_gauss(sig_2,work2,f/2.35482,ntot,.true.)
 
C-----------------------------------------------------------------------
C   Apply shift by "d" pixels
 
      call shift_spec(work1,work3,ntot,d,.false.)
      call shift_spec(work2,work4,ntot,d,.true.)
 
C-----------------------------------------------------------------------
C   Fill the wavelength work array
 
      do 100 i = n1,n2
         wave1(i-n1+1) = wave(i)
 100  continue
 
C-----------------------------------------------------------------------
C   Find chisq at three positions r-dr,r and r+dr
 
 1    do 200 j = 1,3
         ratio = r+(j-2)*dr

C-----------------------------------------------------------------------
C   Fill the arrays wave1,diff and sigma with wavelengths, difference
C   and sigma's resp. for the required wavelength range
 
         do 210 i = n1,n2
            diff(i-n1+1) = flux_1(i) - ratio*work3(i)
            sigma(i-n1+1) = sqrt(sig_1(i)**2+(ratio*work4(i))**2)
 210     continue
 
C-----------------------------------------------------------------------
C   Do the polynomial fitting and calculate chisq
 
         call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
         chi(j) = chisq
 200  continue
 
C-----------------------------------------------------------------------
C   Find minimum in chisq
 
 2    count = count + 1
C*       write(6,98) ratio,chi(1),chi(2),chi(3)
C* 98    format(' Scanning R: Ratio = ',f10.5,' Chisq = ',3f12.2)
      if (count .ge. MAX_ITER) then
         rnew = ratio
         goto 4
      end if
      if (chi(2) .le. chi(1)) then
         if (chi(3) .lt. chi(2)) then
 
C-----------------------------------------------------------------------
C   Chisq decreases with increasing ratio
 
            chi(1) = chi(2)
            chi(2) = chi(3)
            ratio = ratio+dr
            do 300 i = n1,n2
               diff(i-n1+1) = flux_1(i) - ratio*work3(i)
               sigma(i-n1+1) = sqrt(sig_1(i)**2+(ratio*work4(i))**2)
 300        continue
            call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
            chi(3) = chisq
            goto 2
         else
 
C-----------------------------------------------------------------------
C   Minimum found !!!

            goto 3
         end if
      else
         if (chi(3) .ge. chi(2)) then
 
C-----------------------------------------------------------------------
C   Chisq increases with decreasing ratio
 
            chi(3) = chi(2)
            chi(2) = chi(1)
            ratio = ratio - 3*dr
            do 400 i = n1,n2
               diff(i-n1+1) = flux_1(i) - ratio*work3(i)
               sigma(i-n1+1) = sqrt(sig_1(i)**2+(ratio*work4(i))**2)
 400        continue
            call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
            chi(1) = chisq
            ratio = ratio + 2*dr
            goto 2
         else
 
C-----------------------------------------------------------------------
C   Oops, local maximum in chisq. Choose the direction of the lowest and
C      start from the beginning.
 
            if (chi(3) .lt. chi(1)) r = r + dr
            if (chi(3) .gt. chi(1)) r = r - dr
            goto 1
         end if
      end if
 
C-----------------------------------------------------------------------
C   Calculate the actual position of the minimum in chisq by a parabolic
C   fit to the last three values in chi (Bevington, p210).
C   (Where a(3) = ratio)
 
 3    rnew = ratio - dr * (0.5 + (chi(3)-chi(2))/
     &                    (chi(3)-2*chi(2)+chi(1)))
 
C-----------------------------------------------------------------------
C   Calculate the actual minimum chisq
 
 4    do 500 i = n1,n2
         diff(i-n1+1) = flux_1(i) - rnew*work3(i)
         sigma(i-n1+1) = sqrt(sig_1(i)**2+(rnew*work4(i))**2)
 500  continue
      call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)

C*       write(6,99) rnew, chisq
C* 99    format(' Found minimum chisq in R: Ratio = ',f10.5,' Chisq = ',
C*    >               f12.2)
 
      return
      end

C---------------------------------------------------------------------------

      subroutine fitpoly (x,y,sigma,n,coeff,mpoly,chisq)
 
      integer n,mpoly,i

C  Maximum order of fitting polynomial +1

      integer MMAX
      parameter (MMAX=10)

      integer lista(MMAX)
      real covar (MMAX,MMAX), chisq
      real x(n), y(n), sigma(n), coeff(mpoly)
      real xc
 
C-----------------------------------------------------------------------
C    Initialize array LISTA
 
      do 100 i = 1, mpoly
         lista(i) = i
 100  continue
 
C-----------------------------------------------------------------------
C    Center x-range to zero (otherwise LFIT may yield incorrect results)
 
      xc = (x(n)+x(1))/2.0
      do 200 i = 1,n
         x(i) = x(i)-xc
 200  continue
 
C-----------------------------------------------------------------------
 
      call lfit(x,y,sigma,n,coeff,mpoly,lista,mpoly,covar,mpoly,chisq)
 
 
C-----------------------------------------------------------------------
C    Restore original x-range
 
      do 300 i = 1,n
         x(i) = x(i)+xc
 300  continue
 
      return
      end
 
C---------------------------------------------------------------------------

      subroutine funcs(x,p,np)

C  Polynomial function for the Numerical Recipes LFIT routine
 
c     implicit none
 
      integer np ,i
      real p(np),x
 
      p(1) = 1.0
      do 100 i = 2,np
         p(i) = p(i-1)*x
 100  continue
      return
      end

C---------------------------------------------------------------------------

      subroutine gaussj(a,n,np,b,m,mp)

C
C   Gauss-Jordan Elimination Routine.  Taken from Numerical Recipes
C   (Price, Flannery, Teukolsky, & Vetterling) Chapter 2.1
C
C   Modified to better handle errors by replacing "pause" statements
C   with explicit crash message, and to make declarations more explicit
C   [RWP/OSU 1992 April 29]
C

      parameter (NMAX = 50)
      real    a(np,np),b(np,mp)
      integer ipiv(NMAX),indxr(NMAX),indxc(NMAX)

      do 11 j = 1, n
         ipiv(j) = 0
 11   continue
      do 22 i = 1, n
         big = 0.
         do 13 j = 1, n
            if (ipiv(j).ne.1) then
               do 12 k = 1, n
                  if (ipiv(k).eq.0) then
                     if (abs(a(j,k)).ge.big) then
                        big = abs(a(j,k))
                        irow = j
                        icol = k
                     end if
                  else if (ipiv(k).gt.1) then
                     print *, 'GAUSSJ: Singular Matrix'
                     print *, 'Program Aborting'
                     stop
                  end if
 12            continue
            end if
 13      continue
         ipiv(icol) = ipiv(icol)+1
         if (irow.ne.icol) then
            do 14 l = 1, n
               dum = a(irow,l)
               a(irow,l) = a(icol,l)
               a(icol,l) = dum
 14         continue
            do 15 l = 1, m
               dum = b(irow,l)
               b(irow,l) = b(icol,l)
               b(icol,l) = dum
 15         continue
         end if
         indxr(i) = irow
         indxc(i) = icol
         if (a(icol,icol) .eq. 0.) then
            print *,'GAUSSJ: Singular Matrix.'
            print *,'Program Aborting'
            stop
         end if
         pivinv = 1./a(icol,icol)
         a(icol,icol) = 1.
         do 16 l = 1, n
            a(icol,l) = a(icol,l)*pivinv
 16      continue
         do 17 l = 1, m
            b(icol,l) = b(icol,l)*pivinv
 17      continue
         do 21 ll = 1, n
            if (ll.ne.icol) then
               dum = a(ll,icol)
               a(ll,icol) = 0.
               do 18 l = 1, n
                  a(ll,l) = a(ll,l)-a(icol,l)*dum
 18            continue
               do 19 l = 1, m
                  b(ll,l) = b(ll,l)-b(icol,l)*dum
 19            continue
            end if
 21      continue
 22   continue
      do 24 l = n, 1, -1
         if (indxr(l).ne.indxc(l)) then
            do 23 k = 1, n
               dum = a(k,indxr(l))
               a(k,indxr(l)) = a(k,indxc(l))
               a(k,indxc(l)) = dum
 23         continue
         end if
 24   continue
      return
      end

C---------------------------------------------------------------------------

      subroutine lfit(x,y,sig,ndata,a,ma,lista,mfit,covar,ncvm,chisq)

C
C   General Linear Least Squares fit using Gauss-Jordan Elimination to
C   solve the normal linear equations of condition.   
C   Taken from Numerical Recipes (Price, Flannery, Teukolsky, & Vetterling)
C   Chapter 14.3
C
C   LFIT is not the best way to approach GLLS in general, but for this
C   application it seems to suffice.  
C
C   Modified for better error handling (replaced "pause" statements) and
C   made declarations more explicit  [RWP/OSU 1992 April 29]
C

      parameter (MMAX = 50)
      real x(ndata), y(ndata), sig(ndata), a(ma)
      real covar(ncvm,ncvm), beta(MMAX), afunc(MMAX)
      integer lista(mfit)
      
      kk = mfit + 1
      do 12 j = 1, ma
         ihit = 0
         do 11 k = 1, mfit
            if (lista(k) .eq. j) ihit = ihit + 1
 11      continue
         if (ihit .eq. 0) then
            lista(kk) = j
            kk = kk + 1
         else if (ihit .gt. 1) then
            print *,'LFIT: Improper Set in LISTA'
            print *,'Program Aborting'
            stop
         end if
 12   continue
      if (kk .ne. (ma + 1)) then
         print *,'LFIT: Improper Set in LISTA'
         print *,'Program Aborting'
         stop
      end if
      do 14 j = 1, mfit
         do 13 k = 1, mfit
            covar(j,k) = 0.
 13      continue
         beta(j) = 0.
 14   continue
      do 18 i = 1, ndata
         call funcs(x(i),afunc,ma)
         ym = y(i)
         if (mfit .lt. ma) then
            do 15 j = mfit + 1, ma
               ym = ym-a(lista(j))*afunc(lista(j))
 15         continue
         end if
         sig2i = 1./sig(i)**2
         do 17 j = 1, mfit
            wt = afunc(lista(j))*sig2i
            do 16 k = 1, j
               covar(j,k) = covar(j,k) + wt*afunc(lista(k))
 16         continue
            beta(j) = beta(j) + ym*wt
 17      continue
 18   continue
      if (mfit .gt. 1) then
         do 21 j = 2, mfit
            do 19 k = 1, j-1
               covar(k,j) = covar(j,k)
 19         continue
 21      continue
      end if
      call gaussj(covar,mfit,ncvm,beta,1,1)
      do 22 j = 1, mfit
         a(lista(j)) = beta(j)
 22   continue
      chisq = 0.
      do 24 i = 1, ndata
         call funcs(x(i),afunc,ma)
         sum = 0.
         do 23 j = 1, ma
            sum = sum + a(j)*afunc(j)
 23      continue
         chisq = chisq + ((y(i)-sum)/sig(i))**2
 24   continue
      call covsrt(covar,ncvm,ma,lista,mfit)
      return
      end

C---------------------------------------------------------------------------
 
      subroutine read_data (wave,flux_1,sigma_1,flux_2,sigma_2,n)
 
c     implicit         none
 
      real             wave(1),flux_1(1),flux_2(1)
      real             sigma_1(1),sigma_2(1)
      integer          n, NMAX, i, n1, n2
 
      character*40     infile
      
      logical          exist
 
      parameter        (NMAX=1024)
 
      real             wave_1(NMAX), wave_2(NMAX)

C-----------------------------------------------------------------------
C   Prompt for filename of first input file
 
 666  exist = .false.
 
      do while (.not.exist)
         write (6,'(" Enter name of first spectrum : ",$)')
         read (5,'(a)') infile
 
         inquire (file=infile,exist=exist)
         if (.not.exist) then
            write (5,*) ' '
            write (5,*) ' *** WARNING ***  File not found '
            write (5,*) ' '
         end if
      end do

      open(40, file=infile, status='old',iostat=ierr)
      if (ierr .ne. 0) then
         print *,'Cannot open file: ',infile
         print *,'Reason: IOSTAT=',ierr
         go to 666
      end if
      do 100 i = 1,NMAX
         read(40,*,end=1) wave_1(i),flux_1(i),sigma_1(i)
 100  continue
 1    n1 = i-1
      close(40)
 
C-----------------------------------------------------------------------
C   Prompt for filename of second input file
 
 667  exist = .false.
 
      do while (.not.exist)
         write (6,'(" Enter name of second spectrum : ",$)')
         read (5,'(a)') infile
         
         inquire (file=infile,exist=exist)
         if (.not.exist) then
            write (5,*) ' '
            write (5,*) ' *** WARNING ***  File not found '
            write (5,*) ' '
         end if
      end do
 
      open(40, file=infile, status='old', iostat=ierr)
      if (ierr .ne. 0) then
         print *,'Cannot open file: ',infile
         print *,'Reason: IOSTAT=',ierr
         go to 667
      end if
      do 200 i = 1,NMAX
         read(40,*,end=2) wave_2(i),flux_2(i),sigma_2(i)
 200  continue
 2    n2 = i-1
      close(40)
 
      if (n1 .ne. n2) then
         print *,'*** READ_DATA: FATAL ERROR ***'
         print *,'Files have different lengths!'
         stop
      end if
 
      n = n1
 
      do 300 i = 1, n
         wave(i) = wave_1(i)
         if (wave_1(i) .ne. wave_2(i)) then
            print *,'*** READ_DATA: FATAL ERROR! ***'
            print *,'Files have different wavelength scales!'
            stop
         end if
 300  continue
 
      return
      end

C---------------------------------------------------------------------------

      subroutine read_init(r0,f0,d0,dr,df,dd,halt)
      
c     implicit none
      
      real r0, f0, d0, dr, df, dd, halt
      
 1    write(6,'(" Enter initial flux ratio: ",$)')
      read(5,*,err=1) r0
      
 2    write(6,'(" Enter initial convolution factor: ",$)')
      read(5,*,err=2) f0
      
 3    write(6,'(" Enter initial wavelength shift: ",$)')
      read(5,*,err=3) d0
      
 4    write(6,'(" Enter stepsize in flux ratio: ",$)')
      read(5,*,err=4) dr
      
 5    write(6,'(" Enter stepsize in convolution factor: ",$)')
      read(5,*,err=5) df
      
 6    write(6,'(" Enter stepsize in wavelength shift: ",$)')
      read(5,*,err=6) dd
      
 7    write(6,'(" Enter stopping criterium: ",$)')
      read(5,*,err=7) halt
      
      return
      end

C---------------------------------------------------------------------------
      
      subroutine read_lamb(wave,ntot,n1,n2)
      
c     implicit none
      integer ntot,n1,n2,i
      real wave(ntot)
      real wave_min,wave_max
      
      write(6,'(" The wavelength range is from ",f9.2," to ",f9.2)')
     &     wave(1),wave(ntot)
      
 1    write(6,'(" Enter required wavelength range: lambda min: ",$)')
      read(5,*,err=1) wave_min
      
 2    write(6,'(" Enter required wavelength range: lambda max: ",$)')
      read(5,*,err=2) wave_max
      
C  Find (approximate) pixel numbers of these wavelengths
      
      n1 = 1
      n2 = ntot
      do 100 i = 1,ntot
         if (wave(i).lt.wave_min) n1 = i
         if (wave(i).le.wave_max) n2 = i
 100  continue
      
      return
      end

C---------------------------------------------------------------------------
      
      subroutine shift_spec(input,output,n,shift,sigma)
      
C  Shifts the input spectrum by "shift" pixels into the output spectrum
C  Shift not necessarily integer number
C  Note that a shift in a flux array is not the same as a shift in an
C  error array. In the latter case we need to propagate the errors of
C  the points used for the interpolation.
      
c     implicit         none
      integer          n,i,round
      real             input(1),output(1),shift
      real             sign1,frac
      logical          sigma
      
      sign1 = sign(1.0,shift)

C  truncation; gives # pixels to hop over

      round = int(abs(shift)) 
      frac  = abs(shift) - round
      
C  flux shifting

      if (.not.sigma) then
         if (sign1.gt.0) then
            do 100 i = 1,n-round-1
               output(i) = (1.0-frac)*input(i+round) +
     &              frac*input(i+round+1)
 100        continue
            do 110 i = n-round,n
               output(i) = input(i)
 110        continue
         else
            do 120 i = round+2,n
               output(i) = (1.0-frac)*input(i-round) +
     &              frac*input(i-round-1)
 120        continue
            do 130 i = 1,round+1
               output(i) = input(i)
 130        continue
         end if

C  sigma shifting

      else
         if (sign1.gt.0) then
            do 200 i = 1,n-round-1
               output(i) =sqrt( ((1.0-frac)*input(i+round))**2 +
     &              (frac*input(i+round+1))**2 )
 200        continue
            do 210 i = n-round,n
               output(i) = input(i)
 210        continue
         else
            do 220 i = round+2,n
               output(i) =sqrt( ((1.0-frac)*input(i-round))**2 +
     &              (frac*input(i-round-1))**2 )
 220        continue
            do 230 i = 1,round+1
               output(i) = input(i)
 230        continue
         end if
      end if
      
      return
      end
      subroutine write_res(r,f,d,chisq)

C    Output the position of the minimum and the value of chisq to terminal
 
c     implicit none
      save
 
      real r, f, d, chisq
 
      write(6,1) r,f,d,chisq
 1    format(' Minimum found: Ratio =',f8.3,'   FWHM =',f6.2,
     &       '  Shift =',f6.2,'   Chisq =',f8.2)
 
c      open (38, file='results', status='append')
      open (38, file='results', status='unknown',
     &      access='sequential', fileopt='eof',iostat=ierr)
      if (ierr .ne. 0) then
         print *,'Cannot open file: results'
         print *,'Reason: IOSTAT=',ierr
         stop
      end if

      write(38,2,iostat=ierr) r,f,d,chisq
 2    format(f9.4,f7.3,f7.3,f9.3)
      if (ierr .ne. 0) then
         print *, 'Cannot write to file: results'
         print *,' Reason: IOSTAT=',ierr
      end if
      close(38)
 
      return
      end

C---------------------------------------------------------------------------
 
      subroutine wrout(wave,flux_1,sig_1,flux_2,sig_2,ntot,r,f,d)
 
      character*40     outfile
 
      logical          exist
 
      real wave(1)
      real flux_1(1), sig_1(1)
      real flux_2(1), sig_2(1)
      real r, f, d
 
      integer ntot,i,NMAX

C   Maximum spectral length 

      parameter (NMAX=1024)
 
      real work1(NMAX), work2(NMAX), work3(NMAX)
      real work4(NMAX), work5(NMAX), work6(NMAX)
 
C-----------------------------------------------------------------------
C   Prompt for filename of output file
 
      exist = .true.
 
      do while (exist)
         write (6,'(" Enter name of output spectrum file : ",$)')
         read (5,'(a)') outfile
 
         inquire (file=outfile,exist=exist)
         if (exist) then
            write (5,*) ' '
            write (5,*) ' *** WARNING ***  Already exists '
            write (5,*) ' '
         end if
      end do
 
C-----------------------------------------------------------------------
C   Shift second spectrum by 'd' pixels
 
      call shift_spec(flux_2,work1,ntot,d,.false.)
      call shift_spec(sig_2,work2,ntot,d,.true.)
 
C-----------------------------------------------------------------------
C   Scale second spectrum with flux factor 'r'

      do 100 i = 1, ntot
         work1(i) = r*work1(i)
         work2(i) = r*work2(i)
 100  continue
 
C-----------------------------------------------------------------------
 
C   Convolve (first/second) file with gaussian FWHM 'f' depending
C   whether f is (negative/positve)
 
      if (f .ge. 0.0) then
         call conv_gauss(work1,work3,f/2.35482,ntot,.false.)
         call conv_gauss(work2,work4,f/2.35482,ntot,.true.)
      else
         call conv_gauss(flux_1,work5,f/2.35482,ntot,.false.)
         call conv_gauss(sig_1,work6,f/2.35482,ntot,.true.)
         do 200 i = 1, ntot
            work3(i) = work1(i)
            work4(i) = work2(i)
 200     continue
      end if
 
C-----------------------------------------------------------------------
C   Open the output file
 
      open (38, file=outfile, recl=133, status='new',iostat=ierr)
      if (ierr .ne. 0) then
         print *,'Cannot open file: ',outfile
         print *,'Reason: IOSTAT=',ierr
         stop
      end if
         
      if (f.ge.0.0) then
         do 300 i = 1,ntot
            write(38,*) wave(i),work3(i),work4(i),work3(i),work1(i),
     &                  flux_1(i)-work3(i),flux_1(i)-work1(i)
 300     continue
      else
         do 310 i = 1,ntot
            write(38,*) wave(i),work3(i),work4(i),work5(i),flux_1(i),
     &                  work5(i)-work3(i),flux_1(i)-work3(i)
 310     continue
      end if

      close(38)
      return
      end
