
      program iraf2diff

c
c  IRAF2DIFF - Convert IRAF format 1-d spectra into 3-column ASCII spectra
c              for analysis with autodiff or diffspec 
c
c  The IRAF 1-d spectrum in converted into a 3-column ASCII data file in
c  the format:
c
c     lambda(1)   flux(1)   sigmaflux(1)
c     lambda(2)   flux(2)   sigmaflux(2)
c       ...         ...         ...
c     lambda(n)   flux(n)   sigmaflux(n)
c
c  this version fudges the one-sigma errors by assuming a fixed fractional 
c  error, sigma(f)/f, for all pixels.
c
c  Command Syntax:
c     
c     iraf2diff irafspec ascspec error [lam1 lam2]
c 
c  where:
c     irafspec  = name of an input IRAF format 1-D spectrum file
c     ascspec   = name of an output 3-column ASCII format spectrum file
c     error     = fractional flux error, sigma(f)/f, to assume for each 
c                 spectrum pixel
c     lam1 lam2 = [optional] only load pixels with lam1 <= lam <= lam2
c                 into the 3-column ASCII file.
c
c  Limits:
c
c     The maximum number of pixels is currently limited to 1024 by the
c     autodiff and diffspec programs.  This may in fact be inordinately
c     large given the problem at hand.  This program, however, does not
c     apply any size restrictions.  Let the user beware.
c
c     The input IRAF spectrum must be wavelength calibrated and have
c     a linear wavelength scale
c
c  History:
c     Based on iraf2ascii by Brad Peterson, in turn based on other stuff
c     like the IRS package routine getiraf.
c
c  1993 Feb 25: Major revision to allow simple batch mode operation inside
c               of shell scripts using Sun's f77 routines for Unix command
c               line parsing [rwp/osu]
c
c ------------------------------
c
c  compile: f77 -o iraf2diff iraf2diff.f /home/keeler/pogge/src/Util/rputil.a \
c               /usr/local/iraf/lib/libimfort.a /usr/local/iraf/lib/libsys.a \
c               /usr/local/iraf/lib/libvops.a /usr/local/iraf/unix/hlib/libos.a
c
c---------------------------------------------------------------------------

      real*4 data(10000)
      integer iaxis(7)
      character*80 errmsg

C   Unix Command Line Argument Processing

      integer iargc
      integer narg

      character*80 irafname
      character*80 outname
      character*40 nstr
      character*40 argv

      real*4 ferr, lams, lame
      logical needfile
      logical needferr
      logical havelam

C  Number parsing stuff

      integer largv
      integer io_lenc
      integer io_getnum
      real*8 dval

C  Process the command line

      narg = iargc()
      if (narg .eq. 0) then
         needfile = .true.
         needferr = .true.
         havelam = .false.
      else if (narg .eq. 1 .or. narg .eq. 4) then
         print *,'Syntax Error - Must have 0, 2, 3, or 5 Arguments'
         go to 666
      else if (narg .eq. 2) then
         call getarg(1,irafname)
         call getarg(2,outname)
         needfile = .false.
         needferr = .true.
         havelam = .false.
      else if (narg .eq. 3) then
         call getarg(1,irafname)
         call getarg(2,outname)
         needfile = .false.
         call getarg(3,nstr)
         largv = io_lenc(nstr)
         ierr = io_getnum(argv(1:largv),dval)
         if (ierr .ne. 0) go to 666
         ferr = sngl(dval)
         needferr = .false.
         havelam = .false.
      else if (narg .eq. 5) then
         call getarg(1,irafname)
         call getarg(2,outname)
         needfile = .false.
         call getarg(3,nstr)
         largv = io_lenc(nstr)
         ierr = io_getnum(argv(1:largv),dval)
         if (ierr .ne. 0) go to 666
         ferr = sngl(dval)
         needferr = .false.
         call getarg(4,nstr)
         largv = io_lenc(nstr)
         ierr = io_getnum(argv(1:largv),dval)
         if (ierr .ne. 0) go to 666
         lams = sngl(dval)
         call getarg(5,nstr)
         largv = io_lenc(nstr)
         ierr = io_getnum(argv(1:largv),dval)
         if (ierr .ne. 0) go to 666
         lame = sngl(dval)
         havelam = .true.
      else if (narg .gt. 5) then
         print *,'Too Many Arguments'
         go to 666
      end if

C  Finished with the command line, get the missing info, if any

      if (needfile) then
         call io_getstr(irafname,' ',
     &        'IRAF 1-D spectrum file (NO .imh!)',ierr)
         call io_getstr(outname,' ',
     &        'ASCII Spectrum File to Create',ierr)
      end if

      if (needferr) call io_getreal(ferr,0.,0.,0.,
     &        'Fractional Flux Error',ierr)

c     read in the iraf file

      call imopen(irafname,1,im,ierr)
      if (ierr.ne.0) then
         l = io_lenc(irafname)
         write(*,*) 'Cannot open IRAF file ',irafname(1:l)
         call imemsg(ierr,errmsg)
         l = io_lenc(errmsg)
         write(*,*) 'Reason: ',errmsg(1:l)
         go to 5000
      end if

      call imgsiz(im,iaxis,naxis,ipix,ierr)
      if (ierr.ne.0) then
         write(*,*) 'Cannot read image parameters'
         call imemsg(ierr,errmsg)
         l = io_lenc(errmsg)
         write(*,*) 'Reason: ',errmsg(1:l)
         go to 5000
      end if

      if (naxis .ne. 1) then
         l = io_lenc(irafname)
         write(*,*) 'Cannot convert ',irafname(1:l),
     &        ' because it is not a 1-D spectrum'
         call imclos(im,ierr)
         go to 5000
      end if

      nx = iaxis(1)
      npts = nx

C  Get the wavelength scale header cards

      call imgkwr(im,'crval1',wstart,ierr)
      if (ierr.ne.0) call imgkwr(im,'w0',start,ierr)
      if (ierr.ne.0) then
         write(*,*) 
     &        'Cannot read starting lambda from the IRAF file header'
         call imemsg(ierr,errmsg)
         l = io_lenc(errmsg)
         write(*,*) 'Reason: ',errmsg(1:l)
         go to 5000
      end if

      call imgkwr(im,'cdelt1',dx,ierr)
      if (ierr.ne.0) call imgkwr(im,'wpc',dx,ierr)
      if (ierr.ne.0) then
         write(*,*) 
     &        'Cannot read the dispersion from the IRAF file header'
         call imemsg(ierr,errmsg)
         l = io_lenc(errmsg)
         write(*,*) 'Reason: ',errmsg(1:l)
         go to 5000
      end if

C  Load the spectrum into the data file

      call imgs1r(im,data,1,nx,ierr)
      if (ierr.ne.0) then
         write(*,*) 'Cannot read the IRAF file'
         call imemsg(ierr,errmsg)
         l = io_lenc(errmsg)
         write(*,*) 'Reason: ',errmsg(1:l)
         go to 5000
      end if

C  Close the IRAF file

      call imclos(im,ierr)

      zeropt = wstart - dx

c  Open the output file

      open(unit=10,name=outname,type='new',iostat=ierr)
      if (ierr .ne. 0) then
         l = io_lenc(outname)
         call io_errmsg(ierr,2,
     &        'Cannot open the ASCII file '//outname(1:l),.false.)
         go to 5000
      end if

C  Do the conversion

      if (havelam) then

         do 101 i = 1, npts
            wavlen = zeropt + dx*float(i)
            if (wavlen .ge. lams .and. wavlen .le. lame) then
               fluxerr = data(i)*ferr
               write(10,'(1x,f7.2,2(2x,1pg14.6))',iostat=ierr) 
     &              wavlen, data(i), fluxerr
               if (ierr .ne. 0) then
                  l = io_lenc(outname)
                  call io_errmsg(ierr,2,
     &                 'Cannot write to file '//outname(1:l),.false.)
                  close(unit=10)
                  go to 5000
               end if
            end if
 101     continue
      else
         do 100 i = 1, npts
            wavlen = zeropt + dx*float(i)
            fluxerr = data(i)*ferr
            write(10,'(1x,f7.2,2(2x,1pg14.6))',iostat=ierr) 
     &           wavlen, data(i), fluxerr
            if (ierr .ne. 0) then
               l = io_lenc(outname)
               call io_errmsg(ierr,2,
     &              'Cannot write to file '//outname(1:l),.false.)
               close(unit=10)
               go to 5000
            end if
 100     continue
      end if

C  Close the file and quit

      close(unit=10)

 5000 stop

C  Parser error traps

 666  write(*,*)'Usage: iraf2diff irafspec ascspec error [lam1 lam2]'

      end
