C
C  Main Program for chi_auto - contains all subroutines for simple
C  debugging
C
C  RWP - 1992 May 19
C
C---------------------------------------------------------------------------

      program chi_auto
 
C  Automatic scaling of spectra assuming constant flux in certain lines.
C
C  Version 1.0, August 1991
C
C  Ernst van Groningen, Ignaz Wanders
C  Uppsala Astronomical Observatory
C  Box 515
C  S-75120 Uppsala
C  Sweden
C     e-mail:
C       21619::LABAN::ERNST (SPAN)  Ernst.van_Groningen@astro.uu.se  (INTERNET)
C       21619::LABAN::IGNAZ (SPAN)  Ignaz.Wanders@astro.uu.se        (INTERNET)
C
C  The programme fits a simple function to the difference between the
C  2 spectra in the specified wavelength range, and calculates the
C  chisq of this fit. By varying the flux factor, wavelength shift and
C  FWHM of the resolution of the second spectrum the minimum chisq is
C  searched for, according to a grid search method (see Bevington, Data
C  reduction and error analysis for the physical sciences, McGraw-Hill,
C  1969, page 208ff). The search is performed twice, either convolving
C  the first or the second spectrum. The lowest value found is used to
C  calculate the output file. The programme stops when the relative change
C  in chisq from one step to the next is less than the "stopping criterium"
C
C  The programme prompts for the following input parameters
C        > reference spectrum with 3 columns, lambda,flux,rms
C        > spectrum to be scaled, same format
C        > start wavelength of fitting region
C        > end wavelength of fitting region
C        > initial estimate of flux scale factor
C        > initial estimate of FWHM of spectral convolution (pixels)
C        > initial estimate of wavelength shift (pixels)
C        > step size in flux factor
C        > step size in FWHM
C        > step size in shift
C        > stopping criterium
C        > name of output file name
C
C  A summary of the results is written to the file 'results' (in
C  routine WRITE_RES). The 4 columns contain:
C        > the flux ratio (i.e. for scaling the second input file)
C        > the FWHM ( if FWHM>0 the second file is convolved, if
C            FWHM<0 the first file)
C        > the wavelength shift (applied on the second file)
C        > the value of chisq in the minimum
C
C  The modified spectra are written to the specified output file which
C  contains seven columns, whose contents depend on the sign of the
C  FWHM (see above)
C
C  For FWHM > 0
C        1. wavelength
C        2. second spectrum: scaled, shifted and convolved
C        3. one sigma rms of column 2
C        4. identical to column 2
C        5. second spectrum: scaled, shifted (not convolved)
C        6. difference between reference spectrum and column 2
C        7. difference between reference spectrum and column 5
C
C  For FWHM < 0
C        1. wavelength
C        2. second spectrum: scaled, shifted (not convolved)
C        3. one sigma rms of column 2
C        4. convolved reference spectrum
C        5. non modified reference spectrum
C        6. difference between column 4 and column 2
C        7. difference between column 5 and column 2
C
C  CPU time usage depends very much on how good the initial guesses are and
C  on the step sizes. Some tests with 'typical' input parameters yielded:
C
C       Apollo workstation  -	DN3550    140 secs
C				DN10000     4 secs
C	VAX 4300		           16 secs
C
C  Modified to be explicitly ANSI-standard Fortran 77  [RWP/OSU 1992 Apr 29]
C
 
c     implicit none
 
      integer nmax, ntot, status

C  pixel numbers of starting and end of fitting region

      integer n1,n2    

C  maximum spectral length 

      parameter        (NMAX=1024)

C  Array of wavelengths
 
      real wave(NMAX)

C  original flux and sigmas of first file

      real flux_1(NMAX), sig_1(NMAX)

C  original flux and sigmas of second file

      real flux_2(NMAX), sig_2(NMAX)

C  starting values for flux ratio, FWHM and lambda shift

      real r0, f0, d0

C  initial step size for these three parameters

      real dr, df, dd

C  results for these three parameters

      real r, f, d

C  stop if delta(chisq) < halt * ntot

      real halt

C  Chi-squared

      real chisq
 
C-----------------------------------------------------------------------
C    Read spectra from disk
 
      call read_data(wave,flux_1,sig_1,flux_2,sig_2,ntot)
 
C-----------------------------------------------------------------------
C    Inquire for lambda region and find start and end pixel for extraction
 
      call read_lamb(wave,ntot,n1,n2)
 
C-----------------------------------------------------------------------
C    Inquire for starting values, initial stepsizes and stopping criterium
 
      call read_init(r0,f0,d0,dr,df,dd,halt)
 
C-----------------------------------------------------------------------
C    Locate the position of the minimum chisq
 
      call find_min(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &              r0,f0,d0,dr,df,dd,halt,r,f,d,chisq,status)
 
 
C-----------------------------------------------------------------------
C    Output the position of the minimum and the value of chisq
 
      call write_res(r,f,d,chisq)
 
C-----------------------------------------------------------------------
C    Output the resulting spectra to disk
 
      call wrout(wave,flux_1,sig_1,flux_2,sig_2,ntot,r,f,d)

      print *,'*** CHI_AUTO Done ***'
      end
