
      program plotdiff

C
C  PLOTDIFF - plot difference spectra derived from AUTODIF
C  
C  Divides the plotting regions into two windows, plotting the scaled 
C  object and reference spectra in the top window, and the difference 
C  spectrum (object* - reference) in the bottom window.  The difference
C  spectrum window is 1/2 as wide vertically as the object/reference
C  spectrum window.  The error bars are plotted for the difference spectrum.
C  
C  Assumes the difference spectrum file is in the ASCII text format generated
C  by AUTODIFF, the OSU version of van Groningen & Wanders' CHI_AUTO program
C  
C  Command Syntax:
C  
C    plotdiff diffile [-tXX|hard]
C  
C  where: 
C    diffile   is the name of a file containing a difference spectrum
C                in "AUTODIFF" 7-column ASCII text format
C    -tXX      Use Lick Mongo device type XX.  Default is 11 (X-Windows)
C                 other typical values are 1 (vt640) and 13 (GraphOn)
C    hard      is an optional keyword indicating that the 
C                 graphics output is to be sent to the default hardcopy 
C                 device rather than the screen
C  
C  -----------------------------
C  
C  R. Pogge
C  OSU Astronomy Dept.
C  1992 May 22
C  pogge@bruno.mps.ohio-state.edu
C  
C  Link with the LickMongo subroutine library, and the iosubs
C  routine in the source/ directory (iosubs.f).  See buildplot
C  for a shell script to do most of the compilation dirty work.
C  
C  Modification History:
C    1992 May 22: new program, including adding function plotset to rputil
C    1992 May 23: fixed multiple ugly bugs [RWP]
C
C---------------------------------------------------------------------------

C   Command Line Variables

      character diffile*80
      integer idev
      logical hard

C   Working Arrays

      integer MAXPIX
      parameter (MAXPIX=1024)

      real*4  lam(MAXPIX)
      real*4  dflx(MAXPIX), dsig(MAXPIX)
      real*4  oflx(MAXPIX), osig
      real*4  rflx(MAXPIX), rsig

C   Working Variables

      real*8  dval
      real*4  xmn, xmx, ymn, ymx, dmn, dmx
      real*4  xs, xe, fs, fe, ds, de
      real*4  yfrac(2)
      integer ierr
      logical havecomm, isok
      character teststr*80

C   External Functions

      integer io_lenc
      integer io_getnum
      integer plotset
      integer fileplot

C   Unix Command Line Argument Processing

      integer   iargc
      integer   narg
      character argv*40

C   Yow!

      havecomm = .false.
      hard = .false.
      idev = 11
      ihard = 5
      yfrac(1) = 1.
      yfrac(2) = 2.

C   Process the command line

      narg = iargc()
      if (narg .eq. 0) then
         havecomm = .false.
      else if (narg .ge. 1 .and. narg .le. 2) then
         havecomm = .true.
         hard = .false.
         call getarg(1,argv)
         diffile = argv
         if (narg .eq. 2) then
            call getarg(2,argv)
            call io_lower(argv,argv)
            if (argv(1:2) .eq. '-t') then
               hard = .false.
               largv = io_lenc(argv)
               ierr = io_getnum(argv(3:largv),dval)
               if (ierr .ne. 0) then
                  print *,'Bad Numerical Argument: ',argv(3:largv)
                  print *,'Must be a valid LickMongo device code'
                  call io_errmsg(0,0,
     &                 'Usage: diffplot file [-tXX|hard]',.true.)
               end if
               idev = nint(dval)
            else if (argv(1:4) .eq. 'hard') then
               hard = .true.
               idev = ihard
            else
               largv = io_lenc(argv)
               print *,'Unknown flag: ',argv(1:largv)
               call io_errmsg(0,0,
     &              'Usage: plotdiff file [-tXX|hard]',.true.)
            end if
         else
            print *,'plotdiff requires at least 2 arguments'
            call io_errmsg(0,0,
     &           'Usage: plotdiff file [-tXX|hard]',.true.)
         end if
      else if (narg .gt. 2) then
         print *,'Too many arguments'
         call io_errmsg(0,0,'Usage: plotdiff file [-tXX|hard]',.true.)
      end if

C   No command line used, prompt for inputs as required

 100  if (.not. havecomm) then
         call io_getstr(diffile,' ',
     &        'Difference Spectrum Filename',ierr)
         call io_upper(diffile,teststr)
         if (teststr .eq. 'ABORT') then
            print *,'PLOTDIFF Aborting'
            stop
         end if
         call io_getlog(hard,'H','S',hard,
     &        'Screen or Hardcopy Plot',ierr)
         if (.not. hard) then
            call io_getint(idev,0,0,idev,
     &           'LickMongo Device Code',ierr)
         else
            idev = ihard
         end if
      end if

C   Initialize the plotting device

      ierr = plotset(idev,hard,.true.,.false.,' ',.true.)
      if (ierr .ne. 0) then
         if (hard) then
            print *,'Cannot open hardcopy device'
         else
            print *,'Invalid plotting device code ',idev
         end if
         print *,'PLOTDIFF aborting'
         stop
      end if

C   Open the difference spectrum file

      open (unit=1, file=diffile, status='old', iostat=ierr)
      if (ierr .ne. 0) then
         call io_errmsg(ierr,2,
     &        'Cannot open difference spectrum file',havecomm)
         go to 100
      end if

C   Read the spectrum data into the working arrays

      i = 0
      do while (ierr .eq. 0)
         i = i + 1
         read(1,*,iostat=ierr) lam(i), dflx(i), dsig(i),
     &        oflx(i), osig, rflx(i), rsig
         if (ierr .gt. 0) then
            close(1)
            call io_errmsg(ierr,2,
     &           'Cannot read difference spectrum file',.true.)
         end if
      end do
      close(1)
      npts = i - 1

C   Find the plotting limits

      xmn = lam(1)
      xmx = lam(npts)
      xs = xmn - 0.05*(xmx-xmn)
      xe = xmx + 0.05*(xmx-xmn)
      ymn = oflx(1)
      ymx = ymn
      dmn = dflx(1)
      dmx = dmn
      do 200 i = 2, npts
         ymn = amin1(ymn,oflx(i),rflx(i))
         ymx = amax1(ymx,oflx(i),rflx(i))
         dmn = amin1(dmn,dflx(i)-dsig(i))
         dmx = amax1(dmx,dflx(i)+dsig(i))
 200  continue
      fs = ymn - 0.1*(ymx-ymn)
      fe = ymx + 0.1*(ymx-ymn)
      ds = dmn - 0.1*(dmx-dmn)
      de = dmx + 0.1*(dmx-dmn)

C   Setup the plotting window

      call submargins(0.,0.)
      call winfrac(yfrac,2,2)
      if (hard) call setlweight(0.5)
	
C   Plot the scaled object and reference spectra.  
C   Hardcopy: Object = histogram (bin plot), Reference = line plot
C   X-Terminal: Object = yellow histogram, Reference = green histogram
C   Monochrome: Object = histogram, Reference = line

      call window(1,2,2)
      call setlim(xs,fs,xe,fe)
      call abox(0,2,0,0)
      if (hard .or. idev .ne. 11) then
         call histogram(lam,oflx,npts)
         call connect(lam,rflx,npts)
      else
         call setcolor(5)
         call histogram(lam,oflx,npts)
         call setcolor(3)
         call histogram(lam,rflx,npts)
         call setcolor(1)
      end if
      call ylabel(8,'F\\d\\gl')
      call tlabel(io_lenc(diffile),diffile)

C   Plot the difference spectrum and its error bars

      call window(1,2,1)
      call setlim(xs,ds,xe,de)
      call abox(1,2,0,0)
      if (idev .eq. 11) call setcolor(5)
      call histogram(lam,dflx,npts)
      call setexpand(0.)
      call errorbar(2,lam,dflx,dsig,npts)
      call errorbar(4,lam,dflx,dsig,npts)
      call setexpand(1.)
      if (idev .eq. 11) call setcolor(1)
      call xlabel(10,'Wavelength')
      if (hard) then
         call ylabel(12,'(Obj\\g-Ref)')
      else
         call ylabel(9,'(Obj-Ref)')
      end if

C   Terminate the plot, spooling to the printer if a hardcopy plot

      if (hard) then
         nvec = fileplot(0)
         if (idev .ne. 5) print *,'Vectors Plotted: ',nvec
      else
         call tidle
         call io_getlog(isok,'Y','N',.true.,'Plot Another',ierr)
         if (isok) then
            havecomm = .false.
            go to 100
         end if
      end if

      print *,'** PLOTDIFF Done'
      stop
      end

C---------------------------------------------------------------------------

      integer function plotset(idev,hard,land,old,epsfile,doerase)

C
C  PLOTSET - Tool to quickly setup LickMongo for plotting
C
C  Arguments:
C     idev (input, i*4): LickMongo terminal code or internal (VISTA-like)
C                        hardcopy device code (see table below)
C     hard (input, logical): .TRUE. if IDEV is a hardcopy device code
C     land (input, logical): if HARD=T, then specifies landscape (.TRUE.)
C                        or portrait (.FALSE.) hardcopy page orientation
C     old (input, logical): Retain old device state if previously called
C     epsfile (input, char): If given, specifies the name of the PostScript
C                        file to create instead of creating a temporary
C                        scratch file.  Only valid for IDEV=5 & HARD=.TRUE.
C     doerase (input, logical):  If .TRUE., erases the graphics device on 
C                        initialization
C
C  Returns an integer status code to indicate success (0) or failure (>0)
C
C  Sets up the graphics device, and if a hardcopy device opens all relevant
C  scratch or plotting files.  Also sets up the default basic color table as
C  follows, usually only visible on X11 (IDEV=11; HARD=.FALSE.) or color
C  PostScript:
C
C     code   RGB  color
C      0     000  Background (black|white)
C      1     111  Foreground (white|black)
C      2     100  Red
C      3     010  Green
C      4     001  Blue
C      5     110  Yellow
C      6     101  Magenta
C      7     011  Cyan
C
C  Valid LickMongo device codes:
C
C  Terminal (HARD=.FALSE.)
C     1 = DEI Retrographics 640 (VT640 or VT100/Retro)
C     3 = Tektronix 4010 Emulator
C    10 = xterm Tek4010 Emulation (X10 or X11 - brain dead)
C    11 = X-Terminal (X11r4 and later, OpenWindows, DecWindows, etc)
C    13 = GraphOn 200 Series
C    14 = GraphOn 140
C
C  Hardcopy (HARD=.TRUE.)
C     1 = Versatec V80
C     2 = Printronix Line Printer
C     3 = Imagen 8/300
C     4 = ** unused **
C     5 = PostScript
C     6 = Tektronix Metafile (VMS Only)
C
C  Based on the VISTA PLOTSETUP subroutine, with my own kinks.
C  R. Pogge
C  OSU Astronomy Dept.
C  1992 May 22 (last modification)
C  pogge@bruno.mps.ohio-state.edu
C
C  NOTE:  Assumes the Lick Observatory flavor of Mongo.
C
C---------------------------------------------------------------------------

C  External Variables

      integer idev
      logical hard, land, old, doerase
      character epsfile*(*)

C  Yow!

      if (hard) then

c  Versatec V80 (note: versatec acts backwards wrt Portrait mode)

         if (idev .eq. 1) then
c            if (land) then
c               call vtswapxy
c            else
c               call vtsetup
c            end if
c            if (.not. old) call reset
c            call setlweight(2.)
c            call fileinit
            print *,'Device 1 unavailable'
            plotset=1

c  Printronix (note: "swapxy" in prswapxy setup is a curious artifact)
        else if (idev .eq. 2) then
c            call prswapxy
c            if (.not. old) call reset
c            call setlweight(1.)
c            call fileinit
            print *,'Device 2 unavailable'
            plotset=1

c  Imagen 8/300
         else if (idev .eq. 3) then
c            if (land) then
c               call imsetup
c            else
c               call imswap
c            end if
c            if (.not. old) call reset
c            call setlweight(2.)
c            call fileinit
            print *,'Device 3 unavailable'
            plotset=1

C  PostScript Metafile
         else if (idev .eq. 5) then
            call psplot(land,epsfile,' ')
            if (.not. old) call reset
            call setpsfmode(2)
            call setlweight(0.5)

C  Tektronix Metafile (VMS only)
         else if (idev .eq. 6) then
c            call device(6)
c            call tsetup
c            if (.not. old) call reset
c            call setlweight(1.)
            print *,'Device 3 unavailable'
            plotset=1

         else
            print *,'**ERROR: Unknown Hardcopy Device Code ', idev
            plotset = 1
            return
         end if

C  Initialize Graphics Display Device

      else
         call device(idev)
         call tsetup
         if (.not. old) call reset
         if (doerase) call erase
         call setlweight(1.)
         call makecolor(2,1.,0.,0.)
         call makecolor(3,0.,1.,0.)
         call makecolor(4,0.,0.,1.)
         call makecolor(5,1.,1.,0.)
         call makecolor(6,1.,0.,1.)
         call makecolor(7,0.,1.,1.)
      end if

C  Indicate Success and Return

      plotset = 0
      return
      end
