
      program autodiff
C
C  AUTODIFF:  Automatic scaling of spectra assuming constant flux 
C             in certain spectral regions.
C
C  Version 1.1.OSU May 1992 [RWP & BMP]
C
C  We call our program AUTODIFF to distinguish it from the original
C  CHI_AUTO developed by Ernst van Groningen and Ignaz Wanders (see below)
C
C  Comments below on the operation of this program are slightly edited 
C  from the comments in the original chi_auto program.  Except for a 
C  few minor modifications required to ensure proper compilation and
C  execution under ANSI-standard f77 (in particular, Sun f77 v1.4),
C  most of the source code was left untouched.  Most of the modification
C  has taken place in I/O routines (to accomodate local formats) and
C  some user I/O routines to help ultimately streamline user input or
C  batch-mode operation.
C
C  Based on the original CHI_AUTO Version 1.0 
C  Ernst van Groningen, Ignaz Wanders
C  Uppsala Astronomical Observatory
C  Box 515
C  S-75120 Uppsala
C  Sweden
C     e-mail:
C       21619::LABAN::ERNST (SPAN)  Ernst.van_Groningen@astro.uu.se  (INTERNET)
C       21619::LABAN::IGNAZ (SPAN)  Ignaz.Wanders@astro.uu.se        (INTERNET)
C
C  This program fits a simple function to the difference between two
C  spectra in the specified wavelength range, and calculates the
C  Chi-Squared goodness of fit estimator.   By varying a flux scaling
C  factor, the amount of wavelength shift between the object and reference
C  spectrum, and smoothing either the object or reference spectrum by
C  a variable width Gaussian kernel to take into account differences in
C  spectral resolution, the minimum Chi-Squared is determined using
C  a conventional grid search algorithm (see Bevington, Data Reduction and 
C  Error Analysis for the Physical Sciences, McGraw-Hill, 1969, page 208ff). 
C  The search is performed twice, either smoothing (i.e., degrading the
C  resolution of) the reference spectrum or the object spectrum.  The 
C  lowest value found is used to calculate the output file.  The program 
C  stops when the relative change in Chi-Squared from one step to the next 
C  is less than the "stopping criterion" provided by the user.
C
C  The program prompts for the following input parameters in order.  
C  Typical values, where appropriate, are given in []'s:
C        > "reference" spectrum file in 3 column ascii format:
C              lambda, flux, rms
C        > "object" spectrum to be scaled, same file format
C        > starting wavelength of fitting region
C        > ending wavelength of fitting region
C        > initial estimate of the flux scale factor  
C        > initial estimate of the FWHM (in pixels) of the spectral Gaussian 
C            smoothing kernel  [1.0]
C        > initial estimate of the wavelength shift (in pixels)  [0.0]
C        > Grid search step size for the flux factor [0.01]
C        > Grid search step size for the smoothing kernel FWHM  [0.01]
C        > Grid search step size for the wavelength shift  [0.001]
C        > Stopping (convergence) criterion [0.0001]
C
C  When the computation is completed, it will also ask for
C        > name of an output file to hold the difference spectrum
C           and modified reference and object spectrum fluxes & sigmas
C
C  A summary of the results is written to the file 'results' (in
C  routine WRITE_RES). The 4 columns contain:
C        > the Flux Ratio (i.e. for scaling the object spectrum)
C        > the Gaussian smoothing kernel FWHM.
C            if FWHM>0 the object spectrum was smoothed
C            if FWHM<0 the reference spectrum was smoothed
C        > the Wavelength Shift (applied to the object spectrum)
C        > the value of Chi-Squared found when the stopping criterion
C            was satisfied
C  The "results" file is updated with each execution of AUTODIFF, and
C  thus is a running record of the basic results of all difference
C  spectrum runs since the results file was created.
C
C  The modified spectra are written into a user-named output file which
C  contains seven columns.  The contents of these columns depend somewhat
C  on the sign of the FWHM (see above).  Unlike the original CHI_AUTO
C  program, we have adopted our own, more-or-less uniform output format.
C
C  For FWHM > 0
C        1. wavelength
C        2. flux difference spectrum (scaled/smoothed/shifted obj)-(ref)
C        3. one sigma rms of the difference spectrum
C        4. the fluxes of the reference spectrum (unmodified for FWHM>0)
C        5. the sigmas of the reference spectrum (unmodified for FWHM>0
C        6. the fluxes object spectrum fluxes (scaled/smoothed/shifted)
C        7. the sigmas of the modified object spectrum
C
C  For FWHM < 0
C        1. wavelength
C        2. flux difference spectrum (scaled/shifted obj)-(smoothed ref)
C        3. one sigma rms of the difference spectrum
C        4. the fluxes of the SMOOTHED reference spectrum
C        5. the sigmas of the SMOOTHED reference spectrum
C        6. the fluxes object spectrum fluxes (scaled & shifted)
C        7. the sigmas of the modified object spectrum
C
C  Modified to implement features of interest to the AGN Watch "group" at
C  OSU, as well as to make the original CHI_AUTO source code compatible
C  with the Sun f77 version 1.4
C
C  R. Pogge
C  OSU Astronomy Dept
C  pogge@bruno.mps.ohio-state.edu
C  1992 May 15
C  
C  Modification History:
C     1992 May 15:  First pass over original CHI_AUTO code to ensure
C                     compatibility with Sun f77 v1.4 ("ANSI" f77).
C                     The original used many Apollo, VMS, & f90 conventions.
C     1992 May 19:  First test runs: corrected error in array declaration
C                     in FITPOLY subroutine that made f77v1.4 unhappy
C     1992 May 20:  Made all I/O using the default trapping utility 
C                     routines (rputil library)
C                   Made all Fortran error traps give explicit info
C                   Added flux autoscaling feature
C                   Changed output format to a uniform format useful
C                     to us at OSU
C
C--------------------------------------------------------------------------- 
 
      integer nmax, ntot, status

C  pixel numbers of starting and end of fitting region

      integer n1,n2    

C  maximum spectral length 

      parameter (NMAX=1024)

C  Array of wavelengths
 
      real wave(NMAX)

C  original flux and sigmas of first file

      real flux_1(NMAX), sig_1(NMAX)

C  original flux and sigmas of second file

      real flux_2(NMAX), sig_2(NMAX)

C  starting values for flux ratio, FWHM and lambda shift

      real r0, f0, d0

C  initial step size for these three parameters

      real dr, df, dd

C  results for these three parameters

      real r, f, d

C  stop if delta(chisq) < halt * ntot

      real halt

C  Chi-squared

      real chisq

C  Flux autoscaling factor
      
      real fsc

C-----------------------------------------------------------------------
C    Read spectra from disk - OSU version does autoscaling of the 
C    fluxes
C
 
c      call read_data(wave,flux_1,sig_1,flux_2,sig_2,ntot)
      call rd_data2(wave,flux_1,sig_1,flux_2,sig_2,fsc,ntot)

      print *,'Scale Factor Found = ',fsc
C-----------------------------------------------------------------------
C    Inquire for lambda limits of the fitting region and find starting &
C    ending pixel for extraction
 
      call read_lamb(wave,ntot,n1,n2)
 
C-----------------------------------------------------------------------
C    Inquire for initial guesses, grid search step sizes and the
C    grid search stopping criterion
 
      call read_init(r0,f0,d0,dr,df,dd,halt)
 
C-----------------------------------------------------------------------
C    Locate the position of the minimum chisq
 
      call find_min(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &              r0,f0,d0,dr,df,dd,halt,r,f,d,chisq,status)
 
 
C-----------------------------------------------------------------------
C    Output the parameters (scale, smooth, and shift) that give a
C    minimum value of Chi-Squared, and chisq itself into a cumulative
C    results file called "results" in the user's current working directory
 
      call write_res(r,f,d,chisq)
 
C-----------------------------------------------------------------------
C    Output the resulting spectra to disk - OSU version undoes the
C    autoscaling of subroutine RD_DATA2, and writes out the data in
C    a uniform format which does not depend on the sign of F 
 
c      call wrout(wave,flux_1,sig_1,flux_2,sig_2,ntot,r,f,d)

      call wrout2(wave,flux_1,sig_1,flux_2,sig_2,ntot,r,f,d,fsc)

      print *,'*** AUTODIFF Done ***'
      end

