
      program diffspec

C
C  DIFFSPEC:  Batch mode version of CHI_AUTO
C
C  This program is a custom driver for the CHI_AUTO program.  It runs as 
C  a command-line driven shell command, taking arguments off the command 
C  line.  Basic parameters are read from a "config" file.  The use of
C  the command-line syntax is to facilitate batch operation within shell
C  scripts for applications which require repetitive exectution.
C
C  See the comments at the front of autodiff.f for an explanation of
C  the program input and output and the basic algorithm.
C
C  Command Syntax:
C     diffspec [objspec refspec difspec [config]]
C
C  where:
C     objspec = name of the input file with the object spectrum
C     refspec = name of the input file with the reference spectrum
C     difspec = name of the output file with the autodiff output
C      config = name of the config file (config) to read with the 
C                differencing parameters.
C
C  If diffspec is issued without arguments, it will prompt for all
C  input.  If no config file is given, it prompts for the necessary
C  search parameters.  Must have 0, 3, or 4 arguments
C
C  Format of the configuration file
C     
C     lambda1  lambda2
C     scale    scale_step
C     smooth   smooth_step
C     shift    shift_step
C     halt
C
C  where:
C     (lambda1,lambda2) = starting and ending wavelengths of the fitting
C                         region.  lambda1 < lambda2
C     scale = initial estimate of the flux scaling factor   [1.0]
C     scale_step = flux scaling step size [0.01]
C     smooth = initial estimate of the smoothing kernel width  [1.0]
C     smooth_step = smoothing width step size [0.01]
C     shift = initial estimate of the wavelength shift  [0.0]
C     shift_step = shift step size [0.001]
C     halt = stopping criterion  [0.00001]
C
C   Typical default values are in []'s.  All numbers must be REAL
C
C  R. Pogge
C  OSU Astronomy Dept
C  pogge@bruno.mps.ohio-state.edu
C  1992 Sept 8
C  
C--------------------------------------------------------------------------- 
 
      integer nmax, ntot, status

C  wavelengths & pixel numbers of starting and end of fitting region
      
      real*4 wave1, wave2
      integer n1, n2    

C  maximum spectral length 

      parameter (NMAX=1024)

C  Array of wavelengths
 
      real wave(NMAX)

C  original flux and sigmas of first file

      real flux_1(NMAX), sig_1(NMAX)

C  original flux and sigmas of second file

      real flux_2(NMAX), sig_2(NMAX)

C  starting values for flux ratio, FWHM and lambda shift

      real r0, f0, d0

C  initial step size for these three parameters

      real dr, df, dd

C  results for these three parameters

      real r, f, d

C  stop if delta(chisq) < halt * ntot

      real halt

C  Chi-squared

      real chisq

C  Flux autoscaling factor
      
      real fsc

C   Unix Command Line Argument Processing

      integer   iargc
      integer   narg

      character reffile*40
      character objfile*40
      character diffile*40
      character confile*40

      logical havecomm
      logical haveconf

C  Process the command line

      narg = iargc()
      if (narg .eq. 0) then
         havecomm = .false.
         haveconf = .false.
      else if (narg .eq. 1 .or. narg .eq. 2) then
         print *,'Syntax Error - Must have 0, 3, or 4 Arguments'
         go to 666
      else if (narg .gt. 4) then
         print *,'Too Many Arguments'
         go to 666
      else if (narg .eq. 3 .or. narg .eq. 4) then
         havecomm = .true.
         call getarg(1,objfile)
         call getarg(2,reffile)
         call getarg(3,diffile)
         if (narg .eq. 4) then
            haveconf = .true.
            call getarg(4,confile)
         else
            haveconf = .false.
         end if
      end if

C  Load the object & reference spectra into the working wavelength, flux
C  and sigma arrays.

      call read_spec(objfile,reffile,wave,flux_1,sig_1,
     &                   flux_2,sig_2,fsc,ntot,havecomm)

C  Load the fitting parameters from a diffspec configuration file if 
C  specified, otherwise prompt standard output and get all values from 
C  standard input

      if (haveconf) then
         open (unit=1, file=confile, status='old', iostat=ierr)
         if (ierr .ne. 0) call io_errmsg(ierr,2,
     &           'Cannot open the configuration file',.true.)
         read(1,*,iostat=ierr) wave1, wave2
         read(1,*,iostat=ierr) r0, dr
         read(1,*,iostat=ierr) f0, df
         read(1,*,iostat=ierr) d0, dd
         read(1,*,iostat=ierr) halt
         if (ierr .ne. 0) call io_errmsg(ierr,2,
     &           'Cannot read the configuration file',.true.)
         close(1)
         dl = wave(2) - wave(1)
         n1 = nint(1. + (wave1 - wave(1))/dl)
         n2 = nint(1. + (wave2 - wave(1))/dl)
         n1 = min(ntot,max(1,n1))
         n2 = min(ntot,max(1,n2))
      else
         call read_lamb(wave,ntot,n1,n2)
         call read_init(r0,f0,d0,dr,df,dd,halt)
      end if

C End of Parameter Input
C------------------------

C  Locate the position of the minimum chisq via a grid-search
 
      call find_min(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &              r0,f0,d0,dr,df,dd,halt,r,f,d,chisq,status)
 
C  Write the parameters (scale, smooth, and shift) that give a minimum 
C  value of Chi-Squared, and chisq itself into a cumulative results file 
C  called "results" in the user's current working directory
 
      call write_res(r,f,d,chisq)
 
C  Write the resulting scaled and difference spectra disk

      call wd_out(diffile,wave,flux_1,sig_1,flux_2,sig_2,
     &            ntot,r,f,d,fsc,havecomm)

      stop

C  Come here for errors

 666  print *,'Usage: diffspec [objspec refspec difspec [config]]'
      stop
      end

