C
C   iosubs - a collection of program prompting and string parsing routines
C
C     IO_LOWER:   transform all uppercase characters to lowercase
C     IO_UPPER:   transform all lowercase characters to uppercase
C     IO_REMSPC:  remove embedded spaces from a string
C     IO_REM2SPC: replace embedded double spaces in a string with single spaces
C     IO_REMNULL: replace nulls with spaces in a string
C     IO_REMBSPC: remove spaces between []s in a string
C     IO_LENC:    find the length of a string (excluding trailing spaces/nulls)
C     IO_GETREAL: get a real number (R*4)
C     IO_GETDP:   get a double precision real number (R*8)
C     IO_GETINT:  get a long integer (I*4)
C     IO_GETI2:   get a short integer (I*2)
C     IO_GETLOG:  get a logical (L*4) given a simple T/F decision
C     IO_GETSTR:  get a character string
C     IO_GETNUM:  string-to-number converter (used by above)
C     IO_ERRMSG:  print a formatted error message, interpreting codes
C
C  R. Pogge, OSU Astronomy Dept.
C  pogge@astronomy.ohio-state.edu
C  1990 March 12
C
C  Largely based on routines originally provided by Mark Cornell,
C  UT Austin/McDonald Observatory, who got them from someone else long ago.
C
C  Modification History:
C     1990 December 7: changed LENC -> IO_LENC to avoid namespace collision
C                      with the LickMongo primitive.
C     2000 July 1: minor tweaks to aid compilation on a Linux system using
C                  g77 as the fortran compiler (RedHat 6.1 and 6.2 OS)
C
C------------------------------------------------------------------------------

        subroutine io_lower(instr,outstr)

C
C   IO_LOWER:  convert uppercase to lowercase
C
C   Arguments:
C      str (input, char) = string to be converted
C      nchar (output, int) = length of the character string.
C
C   Note: 
C      This routine assumes that there is a constant numerical offset 
C      between the internal representations of a uppercase letter and 
C      its corresponding lowercase letter, and that the numerical 
C      representations are bounded by the relations:
C         value of('A') .LE. value of(any letter) .LE. value of('Z')
C      This is true for ASCII letter representations.
C
C   Modified form of the VISTA routine UPPER by Richard Stover
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

C  External variables

      character*(*) instr
      character*(*) outstr

C  Internal variables

      integer       i
      integer       nchar
      integer       ntrans
      integer       iasc
      character*1   ch1

C  External function

      integer io_lenc

C   Yow!

C   Get number of non-blank characters in the string.

      nchar = io_lenc(instr)
      if (nchar .eq. 0) return

C   Look for uppercase letters and convert them to lowercase.  

      ntrans = ichar('a') - ichar('A')
      do 90010 i = 1, nchar
         ch1 = instr(i:i)
         if (ichar(ch1) .ge. ichar('A') .and. 
     &       ichar(ch1) .le. ichar('Z')) then
            iasc = ichar(ch1) + ntrans
            outstr(i:i) = char(iasc)
         else
            outstr(i:i) = ch1
         end if
90010 continue

      return
      end

C------------------------------------------------------------------------------

        subroutine io_upper(instr,outstr)

C
C   IO_UPPER:  convert lowercase to uppercase
C
C   Arguments:
C      str (input, char) = string to be converted
C      nchar (output, int) = length of the character string.
C
C   Note: 
C      This routine assumes that there is a constant numerical offset 
C      between the internal representations of a lowercase letter and 
C      its corresponding uppercase letter, and that the numerical 
C      representations are bounded by the relations:
C         value of('a') .LE. value of(any letter) .LE. value of('a')
C      This is true for ASCII letter representations.
C
C   Modified form of the VISTA routine UPPER by Richard Stover
C
C   Date of Last Revision:  1990 April 5
C
C------------------------------------------------------------------------------

C  External variables

      character*(*) instr
      character*(*) outstr

C  Internal variables

      integer       i
      integer       nchar
      integer       ntrans
      integer       iasc
      character*1   ch1

C  External function

      integer io_lenc

C   Yow!

C   Get number of non-blank characters in the string.

      nchar = io_lenc(instr)
      if (nchar .eq. 0) return

C   Look for uppercase letters and convert them to lowercase.  

      ntrans = ichar('A') - ichar('a')
      do 90010 i = 1, nchar
         ch1 = instr(i:i)
         if (ichar(ch1) .ge. ichar('a') .and. 
     &       ichar(ch1) .le. ichar('z')) then
            iasc = ichar(ch1) + ntrans
            outstr(i:i) = char(iasc)
         else
            outstr(i:i) = ch1
         end if
90010 continue

      return
      end

C------------------------------------------------------------------------------

      subroutine io_remspc (string, nchars)

C
C   IO_REMSPC:  Remove embedded spaces from a character string
C
C   Arguments:
C      string (i/o, char) = string to be processed
C      nchars (i/o, i*4)  = number of characters
C
C   Based on REMSPACE by Mark Cornell
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

C   External variables

      character*(*) string
      integer       nchars

C   Internal variables

      integer       i, j, nstr

C   External Functions

      integer       io_lenc

C   Yow!

      nstr = io_lenc(string)
      if (index(string, ' ') .eq. 0) return
      j = 0
      do 90010 i = 1, nstr
        if (string(i:i) .ne. ' ' .and. string(i:i) .ne. char(0)) then
          j = j + 1
          string(j:j) = string(i:i)
        end if
90010 continue
      nchars = j
      string(nchars+1:nstr) = ' '
      return
      end

C------------------------------------------------------------------------------

      subroutine io_rem2spc (string, nchars)

C
C   IO_REM2SPC:  Replace embedded double spaces in string with single spaces.
C
C   Arguments:
C      string (i/o, char) = string to be processed
C      nchars (i/o, i*4)  = number of characters
C
C   Based on REM2SPACE by Mark Cornell
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

C   External variables

      character*(*)  string
      integer        nchars

C   Internal variables

      integer    i, j, nstr

C   External functions

      integer    io_lenc

C   Yow!

      nstr = io_lenc(string)
      if (index(string, '  ') .eq. 0) return
      j = 0
      do 90010 i = 1, nstr - 1
        if (string(i:i+1) .ne. '  ' .and. string(i:i) .ne. char(0)) then
          j = j + 1
          string(j:j) = string(i:i)
        end if
90010 continue
      if (string(nstr:nstr) .ne. ' ') then
	j = j + 1
        string(j:j) = string(nstr:nstr)
      end if
      nchars = j
      string(nchars+1:nstr) = ' '
      return
      end

C------------------------------------------------------------------------------

      subroutine io_remnull (string, nchars)

C
C   IO_REMNULL:  Replace embedded nulls in string with spaces.
C
C   Arguments:
C      string (i/o, char) = string to be processed
C      nchars (i/o, i*4)  = number of characters
C
C   Based on REMNULL by Mark Cornell
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

C   External variables

      character*(*)  string
      integer        nchars

C   Internal variables

      integer    i, j
      integer    nstr

C   External functions

      integer    io_lenc

C   Yow!

      nstr = io_lenc(string)
      if (index(string, char(0)) .eq. 0) return
      j = 0
      do i = 1, nstr
        if (string(i:i) .ne. char(0)) then
          j = j + 1
          string(j:j) = string(i:i)
        end if
      end do
      string(j+1:nstr) = ' '
      return
      end

C------------------------------------------------------------------------------

      subroutine io_rembspc (string, nchars)

C
C   IO_REMBSPC:  Removes embedded spaces in a character string between square 
C               and angle brackets.
C
C   Arguments:
C      string (i/o, char) = string to be processed
C      nchars (i/o, i*4)  = number of characters
C
C   Based on REMBSPACE by Mark Cornell
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

C   External variables

      character*(*)  string
      integer        nchars

C   Internal variables

      integer        i, j, nstr
      integer        ntmp
      character*80   tmp

C   External Functions

      integer        io_lenc

C   Yow!

      nstr = io_lenc(string)
      i = index(string, '[')
      if (i .eq. 0) go to 10
      j = index(string(i:nstr), ']')
      if (j .eq. 0) return
      tmp = string(i:i+j-1)
      ntmp = j
      call io_remspc (tmp, ntmp)
      string = string(1:i-1) // tmp(1:ntmp) // string(i+j:nstr)

      nstr = io_lenc(string)
10    i = index(string, '<')
      if (i .eq. 0) return
      j = index(string(i:nstr), '>')
      if (j .eq. 0) return
      tmp = string(i:i+j-1)
      ntmp = j
      call io_remspc (tmp, ntmp)
      string = string(1:i-1) // tmp(1:ntmp) // string(i+j:nstr)
      nchars = nstr - j + ntmp
      return
      end

C-----------------------------------------------------------------------------

	integer function io_lenc(string)

C
C   IO_LENC:  Find the length of the string excluding trailing blanks and nulls
C
C   Arguments:
C      string (input, char) = string to be processed
C
C   Returns the length of the string
C
C------------------------------------------------------------------------------

C   External and Internal variables

      character*(*) string
      integer       i

C   Yow!

      i = len(string)
10    continue
      if (i .gt. 0 .and. (string(i:i) .eq. ' ' .or. 
     &                    ichar(string(i:i)) .eq. 0) ) then
         i = i - 1
         go to 10
      end if
      io_lenc = i
      return
      end

C------------------------------------------------------------------------------

        subroutine io_getreal(value,vmin,vmax,vdef,prompt,ierr)

C
C   IO_GETREAL:  get a real number from user input.
C
C   Issues a prompt on STDOUT for a real number, within a given range,
C   and with a specified default.  Provides error feedback.
C
C   Arguments:
C      value (output, real) = real number received from the user
C      vmin (input, real) = minimum acceptable value
C      vmax (input, real) = maximum acceptable value
C      vdef (input, real) = default value to return
C      prompt (input, char) = character string with input prompt
C      ierr (output, int) = error flag.  0 if no error.
C                       
C   Notes:
C      If VMIN=VMAX, then no range limitations are imposed.
C      Default may be short circuited by setting VDEF outside [VMIN:VMAX] 
C      Non-zero IERR returned on errors are coded according to IO_GETNUM.
C      User has NMAX tries to get input right, otherwise IO_GETREAL exits
C        with IERR=-1.
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

      include 'stdio.inc'

C   External Variables

      real           value
      real           vmin
      real           vmax
      real           vdef
      character*(*)  prompt
      integer        ierr

C   Internal Variables
      
      real*4         tmp
      real*8         dvalue
      character*80   instr
      character*80   defprmt
      logical        range
      logical        default
      integer        lp, ld
      integer        ntries

      integer   NMAX
      parameter (NMAX = 5)

C   External Functions

      integer          io_lenc
      integer          io_getnum

C   Yow!

      range = .true.
      default = .true.
      
C   Check to see if vmin and vmax are inverted

      if (vmin .gt. vmax) then
         tmp = vmin
         vmin = vmax
         vmax = tmp
      end if

C   See if an input range is to be enforced

      if (vmin .eq. vmax) range = .false.

C   See if a default is to be used within an input range

      if (range .and. 
     &    (vdef .lt. vmin .or. vdef .gt. vmax)) default = .false.
      if (.not. range .and. (vdef .eq. vmin)) default = .false.

C   Build the prompt

20    format(a,1pg14.6,a,1pg14.6,a,1pg14.6,a)
21    format(a,1pg14.6,a,1pg14.6,a)
22    format(a,1pg14.6,a)

      if (range .and. default) then
         write (defprmt,20) '<', vmin, ':',vmax, '> [',vdef,'] ? '
      else if (range .and. .not. default) then
         write (defprmt,21) '<', vmin, ':',vmax, '> ? '
      else if (default .and. .not. range) then
         write (defprmt,22) '[',vdef,'] ? '
      else
         defprmt = ' ? '
      end if

      call io_rem2spc (defprmt,ld)
      call io_rembspc (defprmt,ld)
      ld = io_lenc(defprmt)+1
      
C   Issue the prompt and get the input.  The user has NMAX tries to
C   succeed.  This prevents infinite loops if the calling program is
C   being run in batch mode.

      lp = io_lenc(prompt)

      ntries = 0
10    if (ntries .gt. NMAX) then
         call io_errmsg(0,0,'Input Fails',.false.)
         ierr = -1
         return
      end if
      ntries = 1 + ntries

      write (STDERR,'(1x,a,1x,a,$)') prompt(1:lp), defprmt(1:ld)
      read (STDIN,'(a)') instr

C   If INSTR is blank, set value to the default value, and return now

      if (io_lenc(instr) .eq. 0) then
         if (default) then
            value = vdef
            ierr = 0
            return
         end if
         call io_errmsg(0,0,'No default active',.false.)
         go to 10
      end if

C   Otherwise, translate INSTR into a number and test as appropriate
C   The user has NMAX tries to get it right.

      ierr = io_getnum (instr,dvalue)
      if (ierr .ne. 0) then
         call io_errmsg(0,0,'Not a Valid Number',.false.)
         go to 10
      else
         value = sngl(dvalue)
      end if

      if (range) then
         if (value .lt. vmin .or. value .gt. vmax) then
            call io_errmsg(0,0,'Value out of Range',.false.)
            go to 10
         end if
      end if

      ierr = 0
      return
      end

C------------------------------------------------------------------------------

        subroutine io_getdp (dval,dmin,dmax,ddef,prompt,ierr)

C
C   IO_GETDP:  get a double precision real (R*8) from user input.
C
C   Issues a prompt on STDOUT for a double precision number, within a given 
C   range, and with a specified default.  Provides error feedback.
C
C   Arguments:
C      dval (output, r*8) = double precision number received from the user
C      dmin (input, r*8) = minimum acceptable ival
C      dmax (input, r*8) = maximum acceptable ival
C      ddef (input, r*8) = default ival to return
C      prompt (input, char) = character string with input prompt
C      ierr (output, i*4) = error flag.  0 if no error.
C                       
C   Notes:
C      If DMIN=DMAX, then no range limitations are imposed.
C      Default may be short circuited by setting DDEF outside [DMIN:DMAX] 
C      Non-zero IERR returned on errors are coded according to IO_GETNUM.
C      User has NMAX tries to get input right, otherwise IO_GETDP exits
C        with IERR=-1.
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

      include 'stdio.inc'

C   External Variables

      real*8         dval
      real*8         dmin
      real*8         dmax
      real*8         ddef
      character*(*)  prompt
      integer*4       ierr

C   Internal Variables
      
      real*8         dtmp
      character*80   instr
      character*80   defprmt
      logical        range
      logical        default
      integer        lp, ld
      integer*4      ntries

      integer*4 NMAX
      parameter (NMAX = 5)

C   External Functions

      integer*4        io_lenc
      integer*4        io_getnum

C   Yow!

      range = .true.
      default = .true.
      
C   Check to see if dmin and dmax are inverted

      if (dmin .gt. dmax) then
         dtmp = dmin
         dmin = dmax
         dmax = dtmp
      end if

C   See if an input range is to be enforced

      if (dmin .eq. dmax) range = .false.

C   See if a default is to be used within an input range

      if (range .and. 
     &    (ddef .lt. dmin .or. ddef .gt. dmax)) default=.false.
      if (.not. range .and. (ddef .eq. dmin)) default = .false.

C   Build the prompt

20    format(a,1pg14.6,a,1pg14.6,a,1pg14.6,a)
21    format(a,1pg14.6,a,1pg14.6,a)
22    format(a,1pg14.6,a)

      if (range .and. default) then
         write (defprmt,20) '<', dmin, ':',dmax, '> [',ddef,'] ? '
      else if (range .and. .not. default) then
         write (defprmt,21) '<', dmin, ':',dmax, '> ? '
      else if (default .and. .not. range) then
         write (defprmt,22) '[',ddef,'] ? '
      else
         defprmt = ' ? '
      end if

      call io_rem2spc (defprmt,ld)
      call io_rembspc (defprmt,ld)
      ld = io_lenc(defprmt)+1
      
C   Issue the prompt and get the input.  The user has NMAX tries to
C   succeed.  This prevents infinite loops if the calling program is
C   being run in batch mode.

      lp = io_lenc(prompt)

      ntries = 0
10    if (ntries .gt. NMAX) then
         call io_errmsg(0,0,'Input Fails',.false.)
         ierr = -1
         return
      end if
      ntries = 1 + ntries

      write (STDERR,'(1x,a,1x,a,$)') prompt(1:lp), defprmt(1:ld)
      read (STDIN,'(a)') instr

C   If INSTR is blank, set DVAL to the default DVAL, and return now

      if (io_lenc(instr) .eq. 0) then
         if (default) then
            dval = ddef
            ierr = 0
            return
         end if
         call io_errmsg(0,0,'No default active',.false.)
         go to 10
      end if

C   Otherwise, translate INSTR into a number and test as appropriate
C   The user has NMAX tries to get it right.

      ierr = io_getnum (instr,dval)
      if (ierr .ne. 0) then
         call io_errmsg(0,0,'Not a Valid Number',.false.)
         go to 10
      end if

      if (range) then
         if (dval .lt. dmin .or. dval .gt. dmax) then
            call io_errmsg(0,0,'Value out of Range',.false.)
            go to 10
         end if
      end if

      ierr = 0
      return
      end

C------------------------------------------------------------------------------

        subroutine io_getint(ival,imin,imax,idef,prompt,ierr)

C
C   IO_GETINT:  get a long integer (I*4) from user input.
C
C   Issues a prompt on STDOUT for a long integer, within a given range,
C   and with a specified default.  Provides error feedback.
C
C   Arguments:
C      ival (output, i*4) = i*4 number received from the user
C      imin (input, i*4) = minimum acceptable ival
C      imax (input, i*4) = maximum acceptable ival
C      idef (input, i*4) = default ival to return
C      prompt (input, char) = character string with input prompt
C      ierr (output, int) = error flag.  0 if no error.
C                       
C   Notes:
C      If IMIN=IMAX, then no range limitations are imposed.
C      Default may be short circuited by setting IDEF outside [IMIN:IMAX] 
C      Non-zero IERR returned on errors are coded according to IO_GETNUM.
C      User has NMAX tries to get input right, otherwise IO_GETINT exits
C        with IERR=-1.
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

      include 'stdio.inc'

C   External Variables

      integer        ival
      integer        imin
      integer        imax
      integer        idef
      character*(*)  prompt
      integer        ierr

C   Internal Variables
      
      integer        itmp
      real*8         dvalue
      character*80   instr
      character*80   defprmt
      logical        range
      logical        default
      integer        lp, ld
      integer        ntries

      integer   NMAX
      parameter (NMAX = 5)

C   External Functions

      integer          io_lenc
      integer          io_getnum

C   Yow!

      range = .true.
      default = .true.
      
C   Check to see if imin and imax are inverted

      if (imin .gt. imax) then
         itmp = imin
         imin = imax
         imax = itmp
      end if

C   See if an input range is to be enforced

      if (imin .eq. imax) range = .false.

C   See if a default is to be used within an input range

      if (range .and. 
     &    (idef .lt. imin .or. idef .gt. imax)) default=.false.
      if (.not. range .and. (idef .eq. imin)) default = .false.

C   Build the prompt

      if (range .and. default) then
         write (defprmt,*) '<', imin, ':',imax, '> [',idef,'] ? '
      else if (range .and. .not. default) then
         write (defprmt,*) '<', imin, ':',imax, '> ? '
      else if (default .and. .not. range) then
         write (defprmt,*) '[',idef,'] ? '
      else
         defprmt = ' ? '
      end if

      call io_rem2spc (defprmt,ld)
      call io_rembspc (defprmt,ld)
      ld = io_lenc(defprmt)+1
      
C   Issue the prompt and get the input.  The user has NMAX tries to
C   succeed.  This prevents infinite loops if the calling program is
C   being run in batch mode.

      lp = io_lenc(prompt)

      ntries = 0
10    if (ntries .gt. NMAX) then
         call io_errmsg(0,0,'Input Fails',.false.)
         ierr = -1
         return
      end if
      ntries = 1 + ntries

      write (STDERR,'(1x,a,1x,a,$)') prompt(1:lp), defprmt(1:ld)
      read (STDIN,'(a)') instr

C   If INSTR is blank, set ival to the default ival, and return now

      if (io_lenc(instr) .eq. 0) then
         if (default) then
            ival = idef
            ierr = 0
            return
         end if
         call io_errmsg(0,0,'No default active',.false.)
         go to 10
      end if

C   Otherwise, translate INSTR into a number and test as appropriate
C   The user has NMAX tries to get it right.

      ierr = io_getnum (instr,dvalue)
      if (ierr .ne. 0) then
         call io_errmsg(0,0,'Not a Valid Number',.false.)
         go to 10
      else
         ival = nint(dvalue)
      end if

      if (range) then
         if (ival .lt. imin .or. ival .gt. imax) then
            call io_errmsg(0,0,'Value out of Range',.false.)
            go to 10
         end if
      end if

      ierr = 0
      return
      end

C------------------------------------------------------------------------------

        subroutine io_geti2 (ival,imin,imax,idef,prompt,ierr)

C
C   IO_GETI2:  get a short integer (I*2) from user input.
C
C   Issues a prompt on STDOUT for a short integer, within a given range,
C   and with a specified default.  Provides error feedback.
C
C   Arguments:
C      ival (output, i*2) = i*2 number received from the user
C      imin (input, i*2) = minimum acceptable ival
C      imax (input, i*2) = maximum acceptable ival
C      idef (input, i*2) = default ival to return
C      prompt (input, char) = character string with input prompt
C      ierr (output, i*4) = error flag.  0 if no error.
C                       
C   Notes:
C      If IMIN=IMAX, then no range limitations are imposed.
C      Default may be short circuited by setting IDEF outside [IMIN:IMAX] 
C      Non-zero IERR returned on errors are coded according to IO_GETNUM.
C      User has NMAX tries to get input right, otherwise IO_GETI2 exits
C        with IERR=-1.
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

      include 'stdio.inc'

C   External Variables

      integer*2      ival
      integer*2      imin
      integer*2      imax
      integer*2      idef
      character*(*)  prompt
      integer*4       ierr

C   Internal Variables
      
      integer*4      lval
      integer*2      sval

      equivalence(lval,sval)

      integer*2      itmp
      real*8         dvalue
      character*80   instr
      character*80   defprmt
      logical        range
      logical        default
      integer        lp, ld
      integer*4      ntries

      integer*4 NMAX
      parameter (NMAX = 5)

C   External Functions

      integer*4        io_lenc
      integer*4        io_getnum

C   Yow!

      range = .true.
      default = .true.
      
C   Check to see if imin and imax are inverted

      if (imin .gt. imax) then
         itmp = imin
         imin = imax
         imax = itmp
      end if

C   See if an input range is to be enforced

      if (imin .eq. imax) range = .false.

C   See if a default is to be used within an input range

      if (range .and. 
     &    (idef .lt. imin .or. idef .gt. imax)) default=.false.
      if (.not. range .and. (idef .eq. imin)) default = .false.

C   Build the prompt

      if (range .and. default) then
         write (defprmt,*) '<', imin, ':',imax, '> [',idef,'] ? '
      else if (range .and. .not. default) then
         write (defprmt,*) '<', imin, ':',imax, '> ? '
      else if (default .and. .not. range) then
         write (defprmt,*) '[',idef,'] ? '
      else
         defprmt = ' ? '
      end if

      call io_rem2spc (defprmt,ld)
      call io_rembspc (defprmt,ld)
      ld = io_lenc(defprmt)+1
      
C   Issue the prompt and get the input.  The user has NMAX tries to
C   succeed.  This prevents infinite loops if the calling program is
C   being run in batch mode.

      lp = io_lenc(prompt)

      ntries = 0
10    if (ntries .gt. NMAX) then
         call io_errmsg(0,0,'Input Fails',.false.)
         ierr = -1
         return
      end if
      ntries = 1 + ntries

      write (STDERR,'(1x,a,1x,a,$)') prompt(1:lp), defprmt(1:ld)
      read (STDIN,'(a)') instr

C   If INSTR is blank, set ival to the default ival, and return now

      if (io_lenc(instr) .eq. 0) then
         if (default) then
            ival = idef
            ierr = 0
            return
         end if
         call io_errmsg(0,0,'No default active',.false.)
         go to 10
      end if

C   Otherwise, translate INSTR into a number and test as appropriate
C   The user has NMAX tries to get it right.

      ierr = io_getnum (instr,dvalue)
      if (ierr .ne. 0) then
         call io_errmsg(0,0,'Not a Valid Number',.false.)
         go to 10
      else
         lval = nint(dvalue)
         ival = sval
      end if

      if (range) then
         if (ival .lt. imin .or. ival .gt. imax) then
            call io_errmsg(0,0,'Value out of Range',.false.)
            go to 10
         end if
      end if

      ierr = 0
      return
      end

C-----------------------------------------------------------------------------

        subroutine io_getlog (lval,ttag,ftag,ldef,prompt,ierr)

C
C   IO_GETLOG:  get a logical (L*4) from user input.
C
C   Issues a prompt on STDOUT for a logical return, with given T/F values
C   and with a specified default.  Provides error feedback.
C
C   Arguments:
C      lval (output, logical) = T/F logical received from the user
C      ttag (input, char) = identity of "true" condition
C      ftag (input, char) = identity of "false" condition
C      ldef (input, logical) = default condition (T/F)
C      prompt (input, char) = character string with input prompt
C      ierr (output, i*4) = error flag.  0 if no error.
C                       
C   Notes:
C      Input is case insensitive.
C      User has NMAX tries to get input right, otherwise IO_GETLOG exits
C        with IERR=-1.
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

      include 'stdio.inc'

C   External Variables

      logical        lval
      character*(*)  ttag
      character*(*)  ftag
      logical        ldef
      character*(*)  prompt
      integer*4      ierr

C   Internal Variables
      
      character*5    str
      character*1    trustr, falstr
      character*1    dstr
      integer*4      lp
      integer*4      ntries

      integer*4 NMAX
      parameter (NMAX = 5)

C   External functions

      integer io_lenc

C   Yow!

C   Convert T/F strings to lowercase, and set the default string

      call io_lower(ttag,trustr)
      call io_lower(ftag,falstr)

      if (ldef) then
         dstr = trustr
      else
         dstr = falstr
      end if

C   Issue the prompt and get the input.  The user has NMAX tries to
C   succeed.  This prevents infinite loops if the calling program is
C   being run in batch mode.

      lp = io_lenc(prompt)

      ntries = 0
10    if (ntries .gt. NMAX) then
         call io_errmsg(0,0,'Input Fails',.false.)
         ierr = -1
         return
      end if
      ntries = 1 + ntries

      write (STDERR,'(1x,a,'' <'',a1,''|'',a1,''> ['',a1,''] ? ''$)') 
     &       prompt(1:lp), trustr, falstr, dstr
      read (STDIN,'(a)') str

C   If STR is blank, set LVAL to the default, LDEF, and return.

      if (io_lenc(str) .eq. 0) then
         lval = ldef
         ierr = 0
         return
      end if

C   Convert STR to lowercase and test against T/F strings.

      call io_lower(str,str)
      if (str(1:1) .eq. trustr) then 
          lval = .true.
      else if (str(1:1) .eq. falstr) then 
          lval = .false.
      else
         call io_errmsg(0,0,'Invalid Response',.false.)
         go to 10
      end if

      ierr = 0
      return
      end

C------------------------------------------------------------------------------

        subroutine io_getstr (outstr,defstr,prompt,ierr)

C
C   IO_GETSTR:  get a character string from user input.
C
C   Issues a prompt on STDOUT for a character string, with a specified 
C   default.  Provides error feedback.
C
C   Arguments:
C      outstr (output, char) = character string received from the user
C      defstr (input, char) = default string
C      prompt (input, char) = character string with input prompt
C      ierr (output, i*4) = error flag.  0 if no error.
C                       
C   Notes:
C      User has NMAX tries to get input right, otherwise IO_GETSTR exits
C        with IERR=-1.
C      If DEFSTR is blank, no default is assumed
C
C   Date of Last Revision:  1990 March 13
C
C------------------------------------------------------------------------------

      include 'stdio.inc'

C   External Variables

      character*(*)  outstr
      character*(*)  defstr
      character*(*)  prompt
      integer*4       ierr

C   Internal Variables
      
      character*120  instr
      integer*4      lp
      integer*4      ld

C   External functions

      integer io_lenc

C   Yow!

      lp = io_lenc(prompt)
      if (lp .eq. 0) return
      ld = io_lenc(defstr)

C   Issue the prompt and get the input.  

      if (ld .eq. 0) then
         write (STDERR,'(1x,a,'' ? '',$)') prompt(1:lp)

      else if (lp+ld .le. 76) then
         write (STDERR,'(1x,a,'' ['',a,''] ? ''$)') 
     &                           prompt(1:lp), defstr(1:ld)
      else
         write (STDERR,*) prompt
         write (STDERR,'(3x,''['',a,''] ? ''$)') defstr(1:ld)
      end if

      read (STDIN,'(a)') instr

C   If INSTR is blank, set VALUE to the default, LDEF, and return.

      if (io_lenc(instr) .eq. 0) then
         outstr = defstr
      else
         outstr = instr
      end if
      ierr = 0
      return
      end

C------------------------------------------------------------------------------

        integer function io_getnum(instr,value)

C
C   IO_GETNUM:  translate a string into a number
C
C   Converts the string INSTR into a double precision real number VALUE, 
C   returning a status code.
C
C   Arguments:
C
C      instr (input, char) = input character string.  Must represent
C                            a valid Fortran number, including sign and
C                            exponentiation (E).
C
C      value (output, real*8) = real number corresponding to INSTR.
C                        
C   Return Values:
C      io_getnum = 0   Successful Conversion
C      io_getnum = 1   Invalid Character Found in the Mantissa
C      io_getnum = 2   Incomplete exponent (e.g., 1.234e)
C      io_getnum = 3   Invalid character found in the exponent
C      io_getnum = 4   Value out of range (+/-1.0E+38, +/-1.0E-38)
C
C   *** NOTE: IO_GETNUM expects that VALUE is declared as double precision 
C             (REAL*8) in the calling program.
C
C   Based on the routine GETDEC by Richard Stover (from the VISTA package)
C
C   Date of Last Revision:  1990 March 13
C
C   Called by IO_GETREAL and IO_GETINT
C
C------------------------------------------------------------------------------

C   External Variables

      character*(*)     instr 
      double precision  value

C   Internal Variables

      character        c
      double precision fnum
      double precision enum
      double precision esgn
      double precision test
      integer          i
      integer          cval
      integer          nchar
      integer          npoint
      integer          nexp
      logical          expfound
      logical          isreal
      logical          negman
      
C   External Functions

      integer          io_lenc

C   Yow!

      nchar = io_lenc(instr)

C   If NCHAR=1 then INSTR must be a single numeric character to be valid

      if (nchar .eq. 1) then
         c = instr(1:1)
         if ((c .lt. '0') .or. (c .gt. '9')) then
            io_getnum = 1
            return
         end if
      end if

C   Initialize internal variables

      isreal = .false.
      expfound = .false.
      negman = .false.
      nexp = 0
      enum = 0.0
      esgn = 1.0
      fnum = 0.0

C   Decode INSTR character-by-character.  

      do 90010 i = 1, nchar
         c = instr(i:i)
         cval = ichar(c)-ichar('0')

C   Check for a sign at the start of INSTR.  If a - is found, set the
C   negative mantissa flag NEGMAN = .TRUE., otherwise NEGMAN = .FALSE.
C   The plus (+) is understood.

         if (i .eq. 1) then
            if (c .eq. '-') then
               negman = .true.
               go to 90010
            else if (c .eq. '+') then
               goto 90010
            end if
         end if

C   If an exponentiation character ("E") has been found, the rest of
C   INSTR must be the exponent.

         if (expfound) go to 1001

C   INSTR must be decoded differently if an integer instead of floating
C   representation was given.

         if (isreal) goto 1000

C   Check for a decimal point

         if (c .eq. '.') then
            isreal = .true.
            npoint = 1
            go to 90010
         end if

C   Assuming that INSTR represents an integer value (ISREAL=.FALSE.)

         if ((c .ge. '0') .and. (c .le. '9')) then
            fnum = 10.0d0*fnum + cval
            go to 90010
         else if (c .eq. 'e' .or. c .eq. 'E') then
            if (i .lt. nchar) then
               expfound = .true.
               isreal = .true.
               go to 90010
            else
               io_getnum = 2 
               return
            end if
         else
            io_getnum = 1
            return
         end if

C   Assuming that INSTR represents a floating value (ISREAL=.TRUE.)

1000     if ((c .ge. '0') .and. (c .le. '9')) then
            fnum = fnum + dble(cval)*10.0d0**(-npoint)
            npoint = npoint + 1
            go to 90010
         else if (c .eq. 'e'.or. c .eq. 'E') then
            if (i .lt. nchar) then
               expfound = .true.
               isreal = .true.
               go to 90010
            else
               io_getnum = 2 
               return
            end if
         else
            io_getnum = 1
            return
         end if

C   Build up the exponent 

1001     if (nexp .eq. 0) then
            nexp = 1
            if (c .eq. '+') then
               esgn = 1.0
               go to 90010
            else if (c .eq. '-') then
               esgn = -1.0
               go to 90010
            end if
         end if

         if ((c .ge. '0') .and. (c .le. '9') .and. nexp .le. 2) then
            enum = 10.0*enum + cval
            nexp = nexp + 1
            go to 90010
         else
            io_getnum = 3
         end if

90010 continue
      
C   Finish exponential notation if specified.  Watch for fortran 
C   overflows/underflows

      if (expfound) then
         if (fnum .ne. 0.0) then
            test = dlog10(fnum) + esgn*enum
         else
            test = esgn*enum
         end if
         if (abs(test) .le. 38.0 .and. enum .le. 38.0) then
            fnum = fnum*10.0d0**(esgn*enum)
         else
            io_getnum = 4
            return
         end if
      end if

C   Return the value, checking the sign flag

      if (negman) then
         value = -fnum
      else
         value = fnum
      end if

      io_getnum = 0

      return
      end

C---------------------------------------------------------------------------

      subroutine io_errmsg (ierr, isource, errmess, fatal)

C
C   IO_ERRMSG:  Print an Error Message
C
C   Arguments:
C
C      ierr (input, int)      = Error flag passed by some routine
C      errmess (input, char)  = Error message to be displayed to STDERR
C      isource (input, int)   = Flag indicating the error source
C      fatal (input, logical) = Flag to indicate whether error is fatal
C                        
C   Prints the error message ERRMESS.  If the error is fatal (FATAL=.TRUE.) 
C   then the routine halts execution of the calling program.  ISOURCE is a 
C   numerical code giving the source of the error, and if possible, will
C   attempt to translate IERR into something sensible.  Presently recognized 
C   ISOURCE codes are:
C
C      ISOURCE   Action:
C         0       --  Generic error, no translation of IERR
C         1       --  ** unused ** - held for IMFORT??
C         2       --  Fortran IOSTAT, translate using IO_IOERR routine
C         3       --  Warning only (always ignores FATAL)
C         4       --  Abort with message (always fatal)
C
C   R. Pogge
C   1990 March 5
C   OSU Astronomy Dept.
C
C   Date of Last Revision:  1990 April 12
C
C   SunOS Version
C
C------------------------------------------------------------------------------

      include 'stdio.inc'

C   External Variables

      integer       ierr 
      integer       isource
      character*(*) errmess
      logical       fatal

C   Internal variables

c      character*120 errmsg
      integer  k1
c      integer  k2

C   External Functions

      integer  io_lenc

C   Yow!

      k1 = io_lenc(errmess)

      if (isource .eq. 0) then
         write (STDERR,*) '** Error: ',errmess(1:k1)

      else if (isource .eq. 2) then
         write (STDERR,*) '** IOSTAT Error: ',errmess(1:k1)
         call perror('** Reason:')
         ierr = 0

      else if (isource .eq. 3) then
         write (STDERR,*) '** Warning: ',errmess(1:k1)
         return

      else if (isource .eq. 4) then
         write (STDERR,*) '** Abort: ',errmess(1:k1)
         stop

      end if

      if (fatal) then
         write (STDERR,*) '** Fatal Error -- Process Terminated **'
         stop
      end if

      return
      end

